/*
 * classiccounter.cpp
 *
 *  Created on: 21-06-2013
 *      Author: crm
 */

#include "classiccounter.h"

#include "render.h"
#include "graphics/sprite.h"

ClassicCounter::ClassicCounter(): Counter(), spr(), dir(LEFT_TO_RIGHT), prev(0.0f)
	{
	//
	}

ClassicCounter::~ClassicCounter()
	{
	clear();
	}

void ClassicCounter::init(const std::string& name)
	{
	spr.setSprite(name);
	}

void ClassicCounter::update(float dt)
	{
	if(value<min) value=min;
	if(value>max) value=max;
	if(prev>max)  prev=max;

	if(value>=prev)
		prev=value;
	else
		prev-=(max-min)*dt*0.5f;
	}

void ClassicCounter::print(float x, float y)
	{
	const Graphics::Image* img=spr->getImg();
	const Graphics::Animation& a=spr->getAnim(1u);
	const Graphics::Direction& d=a.getDir(Graphics::DEFAULT);
	const Graphics::Frame& f=d.getFrame(0u);

	Render::print(*spr, Orientation(AVector(x, y, 0.0f)), 0u, 0u, 0u);

	if(dir==LEFT_TO_RIGHT)
		{
		const float vfw=f.w*value/(max-min);
		const float pfw=f.w*(prev-value)/(max-min);

		if(prev!=value)
			{
			//Render::setColor(255u, 255u, 255u, 64u);
			Render::print( img, Orientation(AVector(x+vfw-f.spotx, y-f.spoty, 0.0f)), f.x+vfw, f.y, pfw, f.h, 0.25f);
			}

		//Render::setColor(255u, 255u, 255u, 255u);
		Render::print( img, Orientation(AVector(x-f.spotx, y-f.spoty, 0.0f)), f.x, f.y, vfw, f.h, 1.0f);
		}
	else
		{
		const float vfw=f.w*value/(max-min);
		const float pfw=f.w*(prev-value)/(max-min);

		if(prev!=value)
			{
			Render::setColor(255u, 255u, 255u, 64u);
			Render::print( img, Orientation(AVector(x+f.w-vfw-pfw-1-f.spotx, y-f.spoty, 0.0f)), f.x+f.w-vfw-pfw, f.y, pfw, f.h, 0.25f);
			}

		Render::setColor(255u, 255u, 255u, 255u);
		Render::print( img, Orientation(AVector(x+f.w-vfw-0.5f-f.spotx, y-f.spoty, 0.0f)), f.x+f.w-vfw, f.y, vfw, f.h, 1.0f);
		}

	//Render::print(*spr, x, y, 0.0f, 0.0f, 2u, 0u);
	}

void ClassicCounter::clear()
	{
	spr.clear();
	}

int ClassicCounter::getW() const
	{
	return spr.getW();
	}

int ClassicCounter::getH() const
	{
	return spr.getH();
	}
