/*
 * collision.cpp
 *
 *  Created on: 22 maj 2015
 *      Author: crm
 */

#include "collision.h"

#include <algorithm>

#include "log.h"

AVector getSupport(const Box& a, const AVector& dir)
	{
	// Rabniete i przerobione z mojej cudnej magisterki
	const AVector vertices[]=
		{
		AVector(0, 0, 0),
		AVector(a.size.x, 0, 0),
		AVector(a.size.x, a.size.y, 0),
		AVector(0, a.size.y, 0),
		AVector(0, 0, a.size.z),
		AVector(a.size.x, 0, a.size.z),
		AVector(a.size.x, a.size.y, a.size.z),
		AVector(0, a.size.y, a.size.z),
		};

	float best=vertices[0]|dir;
	AVector bestv=vertices[0];

	const unsigned count=sizeof(vertices)/sizeof(vertices[0]);
	for(unsigned i=1u; i<count; ++i)
		{
		AVector v=vertices[i];
		float proj=v|dir;

		if(proj>best)
			{
			best=proj;
			bestv=v;
			}
		}

	return bestv+a.pos;
	}

bool Collision::update(const Box& a, const Box& b)
	{
	using std::min;
	using std::max;

	if(!a.test(b))
		{
		Log::debug("nope");
		return false;
		}

	intersection=Box();

	intersection.pos.x=max(
			min(a.pos.x+a.size.x, b.pos.x),
			min(a.pos.x, b.pos.x+b.size.x));
	intersection.pos.y=max(
			min(a.pos.y+a.size.y, b.pos.y),
			min(a.pos.y, b.pos.y+b.size.y));
	intersection.pos.z=max(
			min(a.pos.z+a.size.z, b.pos.z),
			min(a.pos.z, b.pos.z+b.size.z));
	intersection.pos.w=0.0f;

	intersection.size.x=min(
			max(a.pos.x+a.size.x, b.pos.x),
			max(a.pos.x, b.pos.x+b.size.x)) - intersection.pos.x;
	intersection.size.y=min(
			max(a.pos.y+a.size.y, b.pos.y),
			max(a.pos.y, b.pos.y+b.size.y)) - intersection.pos.y;
	intersection.size.z=min(
			max(a.pos.z+a.size.z, b.pos.z),
			max(a.pos.z, b.pos.z+b.size.z)) - intersection.pos.z;
	intersection.size.w=0.0f;

	Log::info("%f %f %f; %f %f %f", intersection.pos.x, intersection.pos.y, intersection.pos.z, intersection.size.x, intersection.size.y, intersection.size.z);

	const AVector normals[]=
		{
		AVector( 1,  0,  0),
		AVector(-1,  0,  0),
		AVector( 0,  1,  0),
		AVector( 0, -1,  0),
		AVector( 0,  0,  1),
		AVector( 0,  0, -1),
		};

	const unsigned count=sizeof(normals)/sizeof(normals[0]);

	for(unsigned i=0; i<count; ++i)
		{
		//
		}

	return true;
	}


