/*
 * arenadb.cpp
 *
 *  Created on: 25-12-2013
 *      Author: crm
 */

#include "arenadb.h"

#include <vector>
#include <cstdlib>

#include "log.h"
#include "crc32.h"
#include "database.h"
#include "listdir.h"
#include "resfile.h"
#include "resource.h"

#include "gameplay/arena.h"

namespace ArenaDatabase
	{
	Database db;

	std::vector<Entry *> arenas;
	}

namespace ArenaDatabase
	{
	void init(const std::string& path)
		{
		clear();

		std::string file;
		std::vector<std::string> list;

		listdir(path, list);

		for(std::vector<std::string>::iterator it=list.begin(); it!=list.end(); ++it)
			{
			file=*it;

			if(file.size()<3)
				continue;
			if(file.compare(file.size()-3, 3, "crs")!=0)
				continue;

			file=path+((path[path.size()-1]=='/')?"":"/")+file;

			//Log::debug("ArenaDatabase.init: Dodawanie \"%s\"", file.c_str());

			/*Arena c;

			if(!c.init(file+":init.lua"))
				{
				//Log::fail("ArenaDatabase.init: Plik \"%s\" nie zawiera poprawnej postaci", file.c_str());

				c.clear();
				continue;
				}
			else
				{
				Log::debug("ArenaDatabase.init: Wczytano \"%s\"", file.c_str());

				c.clear();
				}*/

			if(getByCRC(crc32(file.c_str())))
				continue;
			if(!Resource::find(file+":init.lua"))
				continue;

			Log::debug("ArenaDatabase.init: Wczytano \"%s\"", file.c_str());

			if(!add(file))
				continue;
			}
		}

	void clear()
		{
		for(std::vector<Entry *>::iterator it=arenas.begin(); it!=arenas.end(); ++it)
			delete *it;
		arenas.clear();

		db.clear();
		}
	}

namespace ArenaDatabase
	{
	bool add(const std::string& path)
		{
		Entry ci;

		if(path.empty())
			return false;
		if(getByPath(path))
			return true;

		Resfile res;

		if(!res.open(path))
			{
			Log::fail("ArenaDatabase.add: Plik \"%s\" nie istnieje lub nie jest poprawnym resfile", path.c_str());
			return false;
			}

		ci.path=path;
		ci.crc=crc32(path.c_str());

		if(ci.crc==0u)
			return false;
		if(getByCRC(ci.crc))
			{
			Log::debug("ArenaDatabase.add: Kolejny plik o tym samym CRC32 (0x%08X), \"%s\"", ci.crc, ci.path.c_str());
			return true;
			}

		arenas.push_back(new Entry(ci));

		return true;
		}

	bool add(unsigned crc, unsigned size, char *data)
		{
		if(size==0)
			return false;
		if(!data)
			return false;
		if(getByCRC(crc))
			{
			Log::debug("ArenaDatabase.add: Kolejny plik o tym samym CRC32 (0x%08X)", crc);
			return true;
			}

		char name[64];
		sprintf(name, "download/lvl_%08X_%08X.crs", crc, rand());

		Log::info("ArenaDatabase.add: Dodawanie areny, sciezka \"%s\"; size %u; crc: 0x%08X", name, size, crc);

		std::fstream out;
		out.open(name, out.out|out.binary|out.trunc);

		if(!out)
			return false;

		out.write(data, size);
		if(out.fail())
			return false;
		out.close();

		return add(name);
		}
	}

namespace ArenaDatabase
	{
	bool load(const std::string& path)
		{
		int count;
		char dpath[16];

		if(!db.load(path))
			return false;

		count=db.getVal("count");

		for(int i=0; i<count; ++i)
			{
			sprintf(dpath, "path_%d", i);

			if(add(db.getStr(dpath)))
				continue;

			Log::fail("ArenaDatabase.load: Nie udalo sie wczytac info o arenie %d/%d; path: \"%s\"", i, count, db.getStr(dpath).c_str());

			db.clear();
			return false;
			}

		db.clear();
		return true;
		}

	bool save(const std::string& path)
		{
		int count;
		char dpath[16];
		//char dorg[16];

		count=arenas.size();

		db.clear();
		db.setVal("count", count);

		for(int i=0; i<count; ++i)
			{
			sprintf(dpath, "path_%d", i);
			//sprintf(dpath, "org_%d", i);

			db.setStr(dpath, arenas[i]->path);
			//db.setStr(dorg, chars[i]->org);
			}

		return db.save(path);
		}
	}

namespace ArenaDatabase
	{
	const Entry* getByCRC(unsigned crc)
		{
		for(std::vector<Entry*>::iterator it=arenas.begin(); it!=arenas.end(); ++it)
			{
			if( (*it)->crc==crc )
				return *it;
			}

		return NULL;
		}

	const Entry* getByPath(const std::string& path)
		{
		for(std::vector<Entry*>::iterator it=arenas.begin(); it!=arenas.end(); ++it)
			{
			if( (*it)->path.compare(path)==0 )
				return *it;
			}

		return NULL;
		}

	/*const ArenaInfo* getByPathOrg(const std::string& org)
		{
		for(std::vector<ArenaInfo*>::iterator it=chars.begin(); it!=chars.end(); ++it)
			{
			if( (*it)->org.compare(org)==0 )
				return *it;
			}

		return NULL;
		}*/

	Arena* createByCRC(unsigned crc, ObjectType::Type type)
		{
		const Entry *ci;
		Arena *c;

		ci=getByCRC(crc);

		if(!ci)
			return NULL;

		c=new Arena();
		c->setType(type);
		c->init(ci->path+":init.lua");

		return c;
		}

	Arena* createByPath(const std::string& path, ObjectType::Type type)
		{
		Arena *c;

		if(!getByPath(path))	// Upewnij sie, ze dana postac jest w db
			return NULL;		// Mozna przeslac patha do inita bez tego, ale zalozmy, ze tego nie zrobimy

		c=new Arena();
		c->setType(type);
		if(!c->init(path+":init.lua"))
			{
			delete c;
			return NULL;
			}

		return c;
		}
	}
