/*
 * controllermanager.cpp
 *
 *  Created on: 03-08-2013
 *      Author: crm
 */

#include "controllermanager.h"

#include <vector>
#include <string>
#include <sstream>

#include "log.h"
#include "databasemanager.h"

#include "controllers/keyboard.h"

//#include "main.h"	// sdl

#include <SDL/SDL.h>

namespace ControllerManager
	{
	std::vector<Controller *> ctrls;

	void initDefault();
	}

void ControllerManager::init(const std::string& name)
	{
	Database& db=DatabaseManager::getDatabase(name);

	int count;

	count=db.getVal("ctrl.count");

	if(count<1)
		{
		initDefault();
		count=db.getVal("ctrl.count");
		}
	if(count>16)
		Log::info("ControllerManager.init: Liczba kontrolerow >16 (%d)", count);

	ctrls.reserve(count);

	std::stringstream ss;

	for(int i=0; i<count; ++i)
		{
		ss.clear();
		ss.str(std::string());

		//ss << "ctrl." << std::setw(2) << std::setfill('0') << i;
		ss << "ctrl." << i;

		std::string type=db.getStr(ss.str());

		ss << "." << type;

		Log::debug("ControllerManager.init: Prefix: \"%s\"", ss.str().c_str());

		if(type.length()<1)
			ctrls.push_back(NULL);
		else if(type.compare("keyboard")==0)
			ctrls.push_back( new ControllerKeyboard(name, ss.str()) );
		else
			Log::fail("ControllerManager.init: Nieznany typ kontrolera: \"%s\" (plik \"%s\")", type.c_str(), name.c_str());
		}
	}

void ControllerManager::update(float dt)
	{
	for(std::vector<Controller *>::iterator it=ctrls.begin(); it!=ctrls.end(); ++it)
		if(*it)
			(*it)->update(dt);
	}

void ControllerManager::clear()
	{
	for(std::vector<Controller *>::iterator it=ctrls.begin(); it!=ctrls.end(); ++it)
		if(*it)
			delete *it;
	ctrls.clear();
	}


void ControllerManager::initDefault()
	{
	Database& db=DatabaseManager::getDatabase("config.nvo");
	//db.clear();
	db.setVal("ctrl.count", 2);

	db.setStr("ctrl.0", "keyboard");
	db.setVal("ctrl.0.keyboard.up", SDLK_UP);
	db.setVal("ctrl.0.keyboard.down", SDLK_DOWN);
	db.setVal("ctrl.0.keyboard.left", SDLK_LEFT);
	db.setVal("ctrl.0.keyboard.right", SDLK_RIGHT);
	db.setVal("ctrl.0.keyboard.a", SDLK_RSHIFT);
	db.setVal("ctrl.0.keyboard.b", SDLK_RCTRL);

	db.setStr("ctrl.1", "keyboard");
	db.setVal("ctrl.1.keyboard.up", SDLK_w);
	db.setVal("ctrl.1.keyboard.down", SDLK_s);
	db.setVal("ctrl.1.keyboard.left", SDLK_a);
	db.setVal("ctrl.1.keyboard.right", SDLK_d);
	db.setVal("ctrl.1.keyboard.a", SDLK_f);
	db.setVal("ctrl.1.keyboard.b", SDLK_g);
	//db.save("config.nvo");
	//db.clear();
	}

Controller* ControllerManager::getController(int id)
	{
	return ctrls[id];
	}

bool ControllerManager::isPressedAny()
	{
	for(std::vector<Controller *>::iterator it=ctrls.begin(); it!=ctrls.end(); ++it)
		{
		Controller *c=*it;

		if(!c)
			continue;

		if(c->isPressed(Controller::UP) ||
		   c->isPressed(Controller::DOWN) ||
		   c->isPressed(Controller::LEFT) ||
		   c->isPressed(Controller::RIGHT) ||
		   c->isPressed(Controller::A) ||
		   c->isPressed(Controller::B))
			return true;
		}

	return false;
	}

bool ControllerManager::isPressed(Controller::Controls c)
	{
	for(std::vector<Controller *>::iterator it=ctrls.begin(); it!=ctrls.end(); ++it)
		if(*it && (*it)->isPressed(c))
			return true;
	return false;
	}
