/*
 * arena.cpp
 *
 *  Created on: 03-09-2013
 *      Author: crm
 */

#include "arena.h"

#include <cstdlib>
#include <cmath>
#include <algorithm>

#include "render.h"

#include "log.h"

#include "consts.h"
#include "clock.h"
#include "luafuncs.h"

#include <gl/glee.h>

const AVector GRAVITY(0.0f, 0.0f, -750.0f, 0.0f);

/*class ArenaCharSort
	{
	public:
		inline bool operator() (Character *c1, Character *c2)
			{
			return (c1->getPos().y<c2->getPos().y);
			}
	};*/

bool Arena::init(const std::string& script)
	{
	clear();

	//path=name;
	path=script.substr(0, script.find_last_of(':', script.length()-1));

	lua.init();
	lua.addRegistry(LUA_PTR_ARENA, this);
	lua.addRegistry(LUA_PTR_GAMEMODE, (void*)NULL);
	lua.addRegistry(LUA_PTR_TYPE, getType());

	const unsigned permissions=LuaFuncs::ARENA|((getType()==OBJECT_MULTIPLAYER_SERVER)?LuaFuncs::MULTIPLAYER_SERVER:LuaFuncs::MULTIPLAYER_LOCAL);

	LuaFuncs::load(lua, permissions|LuaFuncs::INIT);

	if(!lua.open(script))
		return false;

	if(!lua.functionExists(LUA_INIT))
		{
		Log::fail("Arena.init: Nie znaleziono funkcji \"%s\"", LUA_INIT);
		//lua.createFunction(LUA_INIT);
		return false;
		}

	/*if(!lua.functionExists(LUA_UPDATE))
		{
		Log::fail("Arena.init: Nie znaleziono funkcji \"%s\"", LUA_UPDATE);
		lua.createFunction(LUA_UPDATE);
		}*/

	if(objtype!=ObjectType::OBJECT_HEADLESS)
		{
		camera=Render::Camera(camera.PERSPECTIVE, 0, 0, Render::getWindowSize().x, Render::getWindowSize().y, 1.0f, 10000.0f, 60.0f, 1.0f, true);
		}

	lua.call(LUA_INIT);

	LuaFuncs::load(lua, permissions);

	if(!*background && objtype!=OBJECT_HEADLESS)
		background.setSprite("arena/default.crs:bg.xml");

	if(alb.x>art.x)
		{
		float t=alb.x;
		alb.x=art.x;
		art.x=t;
		}

	if(alb.y>art.y)
		{
		float t=alb.y;
		alb.y=art.y;
		art.y=t;
		}

	if(AVectorLength(AVectorSubtract(alb, art)).w<=0.0f)
		{
		alb=AVector(0, 0, 0, 0);
		art=AVector(800, 200, 0, 0);
		}

	if(positions.size()<1)
		{
		positions.push_back(alb+(art-alb)*0.25f);
		positions.push_back(alb+(art-alb)*0.75f);
		}

	Log::ok("Arena.init: Arena \"%s\" wczytana", path.c_str());

	return true;
	}

void Arena::update(float dt)
	{
	/*if(objtype!=ObjectType::OBJECT_NETWORK_CLIENT_SIDE)
		//{
		//Log::info("Arena.update: LUA_UPDATE");
		lua.call(LUA_UPDATE);
		//}*/
	}

void Arena::print()
	{
	if(objtype==ObjectType::OBJECT_HEADLESS)
		return;

	//Render::setCamera(&carena);
	//carena.look(AVector(0, 0, 0), AVector(cos(clock()/1000.0f)*256, sin(clock()/1000.0f)*256, 96), AVector(0, 0, 1));

	Render::setCamera(Render::CAMERA_UI);
	if(*background)
		Render::print(background, Orientation(AVector(0.0f, background.getH(), 0.0f)));

	Render::setCamera(&camera);

	//for(std::vector<Backdrop *>::iterator it=bgsFront.begin(); it!=bgsFront.end(); ++it)
	//	Render::print((*it)->spr, (*it)->pos.x, (*it)->pos.y, 0.0f);	// todo update tla?

	Render::setCamera(Render::CAMERA_DEFAULT);
	}

void Arena::clear()
	{
	/*if(serialized)
		delete [] serialized;
	serialized=NULL;*/

	positions.clear();
	}


void Arena::shake(float force)
	{
	/*for(std::vector<Character *>::iterator it=chars.begin(); it!=chars.end(); ++it)
		{
		Character *c=*it;

		if(c->getPos().z<=0.0f)
			c->jump(rand()/(1.0f+RAND_MAX)*(force*0.75f)+force*0.5f);
		}*/
	}


void Arena::addStartPos(AVector pos)
	{
	positions.push_back(pos);
	}

void Arena::addBackdrop(AVector pos, const std::string& name)
	{
	bgobjs.push_back(new BackgroundObject(pos, /*Orientation(), */name));
	}

void Arena::setSize(float x, float y, float w, float h)
	{
	alb=AVector(x, y, 0, 0);
	art=AVector(x+w, y+h, 0, 0);
	}


bool Arena::getCollision(const AVector& pos) const
	{
	if(	pos.x<alb.x ||
		pos.x>art.x ||
		pos.y<alb.y ||
		pos.y>art.y ||
		pos.z<0.0f)
		return true;
	return false;
	}

/*float Arena::getCollisionDist(const AVector& pos, const AVector& spd) const
	{
	if(spd.x!=0.0f)
		{
		if(spd.x<0.0f)
			return -(pos.x-alb.x);
		else
			return  (art.x-pos.x);
		}
	else if(spd.y!=0.0f)
		{
		if(spd.y<0.0f)
			return -(pos.y-alb.y);
		else
			return  (art.y-pos.y);
		}
	else
		return pos.z;
	}*/


AVector Arena::getGravity() const
	{
	return GRAVITY;
	}
