/*
 * character.cpp
 *
 *  Created on: 23-08-2013
 *      Author: crm
 */

#include "character.h"

#include "log.h"

#include "render.h"

#include "clock.h"
#include "consts.h"
#include "luafuncs.h"
#include "base/gamemode.h"
#include "arena.h"
#include "effects.h"

bool Character::init(const std::string& script)
	{
	clear();

	addEventListener(&combo);

	path=script.substr(0, script.find_last_of(':', script.length()-1));

	lua.init();
	lua.addRegistry(LUA_PTR_CHARACTER, this);
	lua.addRegistry(LUA_PTR_GAMEMODE, gamemode); // Przeciazenie, gdyby nie rzutowanie to potraktuje NULL'a jako inta (a nie wskaznik)
	lua.addRegistry(LUA_PTR_TYPE, getType());

	const unsigned permissions=LuaFuncs::CHARACTER_SELF|((getType()==OBJECT_MULTIPLAYER_SERVER)?LuaFuncs::MULTIPLAYER_SERVER:LuaFuncs::MULTIPLAYER_LOCAL);

	LuaFuncs::load(lua, permissions|LuaFuncs::INIT);

	if(!lua.open(script))
		return false;

	if(!lua.functionExists(LUA_INIT))
		{
		Log::fail("Character.init: [%s]: Nie znaleziono funkcji \"%s\"", script.c_str(), LUA_INIT);
		//lua.createFunction(LUA_INIT);
		return false;	// To akurat jest wazne.
		}
	if(!lua.functionExists(LUA_UPDATE))
		{
		Log::fail("Character.init: [%s]: Nie znaleziono funkcji \"%s\"", script.c_str(), LUA_UPDATE);
		lua.createFunction(LUA_UPDATE);
		}
	if(!lua.functionExists(LUA_HANDLER_CASTED))
		{
		Log::fail("Character.init: [%s]: Nie znaleziono funkcji \"%s\"", script.c_str(), LUA_HANDLER_CASTED);
		lua.createFunction(LUA_HANDLER_CASTED);
		}
	if(!lua.functionExists(LUA_HANDLER_ATTACK))
		{
		Log::fail("Character.init: [%s]: Nie znaleziono funkcji \"%s\"", script.c_str(), LUA_HANDLER_ATTACK);
		lua.createFunction(LUA_HANDLER_ATTACK);
		}
	if(!lua.functionExists(LUA_HANDLER_DEFEND))
		{
		Log::fail("Character.init: [%s]: Nie znaleziono funkcji \"%s\"", script.c_str(), LUA_HANDLER_DEFEND);
		lua.createFunction(LUA_HANDLER_DEFEND);
		}
	if(!lua.functionExists(LUA_HANDLER_HIT))
		{
		Log::fail("Character.init: [%s]: Nie znaleziono funkcji \"%s\"", script.c_str(), LUA_HANDLER_HIT);
		lua.createFunction(LUA_HANDLER_HIT);
		}

	lua.call(LUA_INIT);

	LuaFuncs::load(lua, permissions);

	Log::debug("Character.init: [%s]: Skrypt wykonany", script.c_str());

	if(ideas.size()<1)
		{
		Log::fail("Character.init: [%s]: Postac nie ma zdefiniowanych atakow", script.c_str());
		return false;
		}

	if(!*spr)
		{
		Log::fail("Character.init: [%s]: Postac nie ma zdefiniowanego sprite'a", script.c_str());
		spr.setSprite("character/default.crs:character.xml");
		}

	if(objtype!=OBJECT_HEADLESS)
		{
		if(!*shadow)
			{
			Log::fail("Character.init: [%s]: Postac nie ma zdefiniowanego cienia", script.c_str());
			shadow.setSprite("character/default.crs:shadow.xml");
			}
		}

	//if(!*speech)
	//	speech.setSprite("speech/default.xml");
	if(stats.maxHP<=0.0f)
		stats.maxHP=1.0f;
	if(stats.maxMP<=0.0f)
		stats.maxMP=1.0f;
	//if(stats.speed<=0.0f)
	//	stats.speed=0.1f;
	if(stats.castspeed<=0.0f)
		stats.castspeed=stats.maxMP;
	if(stats.strength<=0.0f)
		stats.strength=1000.0f;

	hp=stats.maxHP;
	mp=stats.maxMP;

	castusage=0.0f;
	castid=0;
	casted=-1;

	idea=NULL;

	block=0.0f;
	castblock=0.0f;

	ref=0;
	luaacc=0.0f;

	Log::debug("Character.init: [%s]: Postac wczytana", name.c_str());

	return true;
	}


void Character::update(float dt)
	{
	//return;

	if(getState(DEAD))
		{
		if(hp>0.0f)
			delState(DEAD);	// Auto reset po 'wskrzeszeniu' na starcie nowej rundy

		updateMove(dt);
		if(objtype!=OBJECT_HEADLESS)
			updateAnim(dt);
		return;
		}

	if(objtype!=OBJECT_MULTIPLAYER_CLIENT)
		{
		if(ref==0)
			{
			spd.x=0.0f;
			spd.y=0.0f;
			}

		luaacc+=dt;
		while(luaacc>LUADELAY)
			{
			luaacc-=LUADELAY;
			if(ref==0 || !getState(COMBO))
				lua.call(LUA_UPDATE);
			else
				lua.call(ref);
			}
		}

	if(objtype==OBJECT_MULTIPLAYER_CLIENT)
		updateClient(dt);
	else if(ref==0)
		updateCtrls(dt);
	updateMove(dt);
	updateMp(dt);

	if(idea)
		{
		idea->setPos(pos+AVector(spr.getActionPointX(), 0, -spr.getActionPointY()));

		//printf("%f %f %f - %f %f %f", pos.x, pos.y, pos.z, idea->getPos().x, idea->getPos().y, idea->getPos().z);
		}

	if(spd.x<0.0f)
		dir=LEFT;
	else if(spd.x>0.0f)
		dir=RIGHT;

	if(objtype!=OBJECT_HEADLESS)
		updateAnim(dt);

	//printf("hp %f air %f use %f block %f spkblock %f ref %d", hp, air, use, block, spkblock, ref);
	}

void Character::updateCtrls(float dt)
	{
	if(getState(DISABLED))
		return;

	if(block>0.0f)
		{
		block-=dt;

		if(block>0.0f)
			return;
		}

	if(castblock>0.0f)
		{
		castblock-=dt;
		}

	/************** Atak *****************************************************/
	if(getState(ATTACKING) && ctrl->isPressed(Controller::ATTACK))
		{
		attack();

		setCastBlock(0.25f);

		return;
		}

	/************** Obrona ***************************************************/
	if(ctrl->isPressed(Controller::DEFEND) && canDefend())
		{
		if(event.type!=Event::CHARACTER_DEFEND)
			{
			pushEvent(Event::CHARACTER_DEFEND);

			if(startCombo(combo.hasMatch()))
				return;

			lua.call(LUA_HANDLER_DEFEND);
			}

		mp-=std::min(stats.maxMP*1.00f*dt, mp);

		defend();
		return;
		}
	else if(getState(DEFENDING))
		{
		pushEvent(Event::CHARACTER_NONE);
		delState(DEFENDING);
		}

	/************** Castowanie ***********************************************/
	const float MP_DIFF=	std::min(stats.castspeed*dt, mp);
	const float MP_MIN=		ideas[0]->getCost();

	if(ctrl->isPressed(Controller::ATTACK) && canCast())
		{
		if(castusage<=0.0f && mp>=MP_MIN)
			{
			pushEvent(Event::CHARACTER_CHARGING);
			if(startCombo(combo.hasMatch()))
				return;

			addState(CASTING);

			castid=0;
			casted=-1;

			castusage+=MP_DIFF;
			mp-=MP_DIFF;
			}
		}
	else if(getState(CASTING))
		{
		if(ctrl->isPressed(Controller::ATTACK))
			{
			if(castusage<ideas[castid]->getCost())
				{
				castusage+=MP_DIFF;
				mp-=MP_DIFF;
				}
			else if(casted!=castid)
				{
				casted=castid;

				pushEvent(Event::CHARACTER_CHARGED, 0.0f, casted);
				if(startCombo(combo.hasMatch()))
					return;	// todo upewnic sie ze wlaczanie combo nie zepsuje castowania

				if((int)ideas.size()>castid+1 && mp>=ideas[castid+1]->getCost())
					{
					castusage-=ideas[castid]->getCost();

					++castid;
					}
				}
			}
		else
			{
			if(casted>=0)
				{
				giveIdeaSelf(casted);

				pushEvent(Event::CHARACTER_CHARGED|Event::CHARACTER_GOT_IDEA, 0.0f, casted, idea);
				if(startCombo(combo.hasMatch()))
					{
					// Jesli ktos zrobil combo z castem, to moze chciec miec pomysl wycastowany
					//delete idea;
					//idea=NULL;

					return;	// todo upewnic sie ze wlaczanie combo nie zepsuje castowania
					}

				addState(ATTACKING);

				lua.push(casted);
				lua.call(LUA_HANDLER_CASTED);
				}
			else
				pushEvent(Event::CHARACTER_MISC, 0.0f, 0, NULL);	// Event aktualizujacy state u klientow w razie nie wycastowania zadnego pomyslu

			delState(CASTING);

			castusage=0.0f;
			castid=0;
			casted=0;
			}
		}

	addState(CONTROLLABLE);

	if(ctrl->isPressed(Controller::LEFT))	moveLeft();
	if(ctrl->isPressed(Controller::RIGHT))	moveRight();
	if(ctrl->isPressed(Controller::UP))		moveUp();
	if(ctrl->isPressed(Controller::DOWN))	moveDown();

	if(AVectorLength(spd).w>0.0f)
		{
		const unsigned e= Event::CHARACTER_MOVE |
			((spd.x<0.0f)*Event::CHARACTER_MOVE_LEFT) |
			((spd.x>0.0f)*Event::CHARACTER_MOVE_RIGHT) |
			((spd.y<0.0f)*Event::CHARACTER_MOVE_DOWN) |
			((spd.y>0.0f)*Event::CHARACTER_MOVE_UP);

		pushEvent(e, 0.0f);

		if(startCombo(combo.hasMatch()))
			return;
		}
	else if(event.event&Event::CHARACTER_MOVE)
		pushEvent(Event::CHARACTER_NONE);
	}

void Character::updateMove(float dt)
	{
	AVector vdt=AVectorReplicate(dt);
	Arena *arena=gamemode->getArena();

	//if(!arena->getCollision(pos, AVectorMultiply(GRAVITY, vdt)))
	if(pos.z>0.0f)	// ^ olac dobre praktyki, poki co chce, zeby to dzialalo :F
		//spd=AVectorAdd(spd, AVectorMultiply(GRAVITY, vdt));
		spd.z+=arena->getGravity().z*dt;

	AVector spddt=AVectorMultiply(spd, vdt);

	/*if(arena->getCollision(pos, spddt))
		{
		const AVector spddtx=AVectorSet(spddt.x, 0.0f, 0.0f, 0.0f);
		const AVector spddty=AVectorSet(0.0f, spddt.y, 0.0f, 0.0f);
		const AVector spddtz=AVectorSet(0.0f, 0.0f, spddt.z, 0.0f);

		if(arena->getCollision(pos, spddtx))	 spddt.x=arena->getCollisionDist(pos, spddtx);
		if(arena->getCollision(pos, spddty))	 spddt.y=arena->getCollisionDist(pos, spddty);
		if(arena->getCollision(pos, spddtz))
			{
			spddt.z=-arena->getCollisionDist(pos, spddtz);

			if(spd.z<arena->getGravity().z*0.3f)
				spd.z*=-0.5f;
			else
				spd.z = 0.0f;
			}
		}*/

	if(AVectorLength(spddt).w>0.0f)
		addState(MOVING);
	else
		delState(MOVING);

	pos=AVectorAdd(pos, spddt);
	}

void Character::updateAnim(float dt)
	{
	if(spr.ended())
		{
		if(getState(DEFENDING))
			spr.setAnim("defending");
		/*else if(getState(ATTACKING))
			spr.setAnim("speaking");*/
		else if(getState(MOVING))
			{
			if(spd.z>0.0f)
				spr.setAnim(0);
			else if(spd.z<0.0f)
				spr.setAnim("falling");
			else
				spr.setAnim("moving");
			}
		else if(getState(DEAD))
			spr.setAnim("dead");
		else
			spr.setAnim(0);
		}

	spr.setDir(dir);

	spr.update(dt);
	shadow.update(dt);
	}

void Character::updateMp(float dt)
	{
	if(!getState(CASTING|DEFENDING|ATTACKING|COMBO))
		{
		if(mp>=stats.maxMP)
			return;

		const float add=stats.maxMP*0.50f*dt;

		if(mp+add<stats.maxMP)
			mp+=add;
		else
			mp=stats.maxMP;
		}
	}

void Character::updateClient(float dt)
	{
	if(getState(DISABLED))
		return;

	if(block>0.0f)
		{
		block-=dt;

		if(block>0.0f)
			return;
		}

	if(castblock>0.0f)
		{
		castblock-=dt;
		}


	/************** Obrona ***************************************************/
	if(getState(DEFENDING))
		{
		mp-=std::min(stats.maxMP*1.00f*dt, mp);

		defend();
		}

	/************** Castowanie ***********************************************/
	const float MP_DIFF=	std::min(stats.castspeed*dt, mp);

	if(getState(CASTING))
		{
		if(castusage<ideas[castid]->getCost())
			{
			castusage+=MP_DIFF;
			mp-=MP_DIFF;
			}
		else
			{
			casted=castid;

			if((int)ideas.size()>castid+1 && mp>=ideas[castid+1]->getCost())
				{
				castusage-=ideas[castid]->getCost();

				++castid;
				}
			}
		}
	}


void Character::print()
	{
	if(objtype==OBJECT_HEADLESS)
		return;

	const float alpha=std::max(0.0f, 1.0f-pos.z/256.0f);
	const float scale=1.0f;//1.0f+(1.0f-alpha)*2;

	Render::print(spr, Orientation(pos, Orientation::FLAT_XZ));
	//Render::print(spr, Orientation(pos, Render::getCamera()->getUp(), -Render::getCamera()->getForward()));
	//Render::print(spr, Orientation(pos, arena->getCamera()->getBillboard(pos)));
	Render::print(shadow, Orientation(AVector(pos.x, pos.y, 0.0f), Orientation::FLAT_XY, AVector(scale, scale, 1.0f)), alpha);
	}


void Character::clear()
	{
	if(serialized)
		delete [] serialized;
	serialized=NULL;

	combo.clear();
	eventListeners.clear();

	ctrl=NULL;
	gamemode=NULL;

	state=NONE;

	id=-1;

	event=Event(0, 0, 0, 0);

	for(std::vector<Idea*>::iterator it=ideas.begin(); it!=ideas.end(); ++it)
		delete *it;

	spr.clear();
	shadow.clear();
	speech.clear();
	ideas.clear();

	sounds.clear();

	name=std::string();
	path=std::string();
	}


char* Character::serialize(int &size)
	{
	size=sizeof(CharacterSerialized);

	if(!serialized)
		serialized=new char[size];

	CharacterSerialized *c=(CharacterSerialized *)serialized;

	c->pos=pos;
	c->spd=spd;
	c->dir=dir;
	c->state=state;
	c->hp=hp;
	c->mp=mp;
	c->pos.w=block;		// miejmy nadzieje, ze to nic nie skopie
	c->spd.w=castblock;

	return serialized;
	}

void printState(unsigned state)
	{
	fprintf(stdout, " > STATE: ");
	if(state&Character::CONTROLLABLE)	fprintf(stdout, "[CONTROLLABLE]");
	if(state&Character::NOCHANGE)		fprintf(stdout, "[NOCHANGE]");
	if(state&Character::MOVING)			fprintf(stdout, "[MOVING]");
	if(state&Character::DEFENDING)		fprintf(stdout, "[DEFENDING]");
	if(state&Character::CASTING)		fprintf(stdout, "[CHARGING]");
	if(state&Character::ATTACKING)		fprintf(stdout, "[SPEAKING]");
	if(state&Character::COMBO)			fprintf(stdout, "[COMBO]");
	if(state&Character::DEAD)			fprintf(stdout, "[DEAD]");
	if(state&Character::DISABLED)		fprintf(stdout, "[DISABLED]");
	if(state==Character::NONE)			fprintf(stdout, "[NONE]");
	fprintf(stdout, "\n");
	}

void Character::deserialize(char *buff)
	{
	CharacterSerialized *c=(CharacterSerialized *)buff;

	/*if(c->hp>hp)
		Log::info("hp miss had %f got %f diff %f", hp, c->hp, c->hp-hp);
	if(c->state!=state)
		{
		Log::info("state miss had 0x%08X got 0x%08X", state, c->state);
		printState(state);
		printState(c->state);
		}*/

	pos=c->pos;
	spd=c->spd;
	dir=c->dir;
	state=c->state;
	hp=c->hp;
	mp=c->mp;
	block=c->pos.w;
	castblock=c->spd.w;
	pos.w=0.0f;
	spd.w=0.0f;
	}


void Character::pushEvent(unsigned e, float dataf, int datai, void *datap)
	{
	Event event=Event(Event::TYPE_CHARACTER, e, Clock::getTimeMilis());

	event.character.who=this;
	event.character.dataf=dataf;
	event.character.datai=datai;
	event.character.datap=datap;

	sendEvent(event);
	}

void Character::sendEvent(const Event& e)
	{
	if(event.event==e.event)
		{
		if(e.event==e.CHARACTER_NONE)
			return;
		if(e.event==e.CHARACTER_DEFEND)
			return;
		if(e.event&e.CHARACTER_MOVE)
			return;
		}

	if(objtype==OBJECT_MULTIPLAYER_SERVER)
		Log::info("Character.pushEvent: %s; %f; %d; 0x%08X", getEventNameCharacter(e.event).c_str(), e.character.dataf, e.character.datai, e.character.datap);

	event=e;
	EventProvider::sendEvent(e);
	}


bool Character::startCombo(int sref)
	{
	if(sref<=0)
		return false;
	ref=sref;
	addState(COMBO);
	pushEvent(Event::CHARACTER_COMBO, 0.0f, ref);
	lua.push(0);
	lua.call(ref);
	return true;
	}


void Character::giveIdeaSelf(int id)
	{
	if(id<0 || id>=(int)ideas.size())
		return;

	if(idea)
		{
		gamemode->delIdea(idea);
		delete idea;
		}

	idea=ideas[id]->clone();
	idea->setOwner(this);
	idea->setGameMode(gamemode);
	gamemode->addIdea(idea);
	}

void Character::giveIdeaOther(int id)
	{
	if(idea)
		{
		gamemode->delIdea(idea);
		delete idea;
		}

	idea=gamemode->getIdeaByID(id);

	if(idea)
		idea->setOwner(this);
	}


void Character::revive()
	{
	spd=AVectorZero();
	state=CONTROLLABLE;
	hp=stats.maxHP;
	mp=stats.maxMP;
	castusage=0.0f;
	block=0.0f;
	castblock=0.0f;
	ref=0;
	event=Event(0, 0, 0, 0);
	}


bool Character::canCast() const
	{
	if(getState(DEFENDING|CASTING|ATTACKING))
		return false;
	if(getCastBlock()>0)
		return false;
	return true;
	}

bool Character::canDefend() const
	{
	if(getState(CASTING|ATTACKING))
		return false;
	if( (mp<=stats.maxMP*0.15f && !getState(DEFENDING)) || (mp<=0.0f) )
		return false;
	return true;
	}


void Character::attack(bool noevent)
	{
	if(!idea)
		return;

	if(!noevent)
		{
		pushEvent(Event::CHARACTER_ATTACK, 0.0f, 0, idea);
		if(startCombo(combo.hasMatch()))
			return;
		}

	idea->setSpd(spd + (dir==LEFT)*AVector(-stats.strength, 0.0f, 0.0f) + (dir==RIGHT)*AVector( stats.strength, 0.0f, 0.0f));
	idea->setOwner(NULL);

	idea=NULL;

	delState(ATTACKING);
	pushEvent(Event::NONE);

	lua.call(LUA_HANDLER_ATTACK); // Musi byc juz po rzuceniu pomyslu - w handlerze mozna zrobic atakowanie, ktore wywola handlera, ktory wywola atak...
	}

void Character::defend()
	{
	addState(DEFENDING);
	delState(CONTROLLABLE);

	pushEvent(Event::CHARACTER_DEFEND);
	}

void Character::hit(float dmg)
	{
	if(state&DEFENDING)
		dmg/=4.0f;

	hp-=dmg;

	//addEvent(Event::HIT | ((this==who)*Event::HIT_SELF));
	pushEvent(Event::CHARACTER_HIT, dmg);
	if(startCombo(combo.hasMatch()))
		return;

	lua.push(dmg);
	lua.call(LUA_HANDLER_HIT);

	if(hp<=0.0f)
		setState(DEAD);

	//Log::info("Character.hit: dmg %f self %d", dmg, who==this);
	}


void Character::moveLeft()
	{
	spd.x+=-(stats.speed * (1.0f - 0.5f*getState(CASTING|ATTACKING)));
	}

void Character::moveRight()
	{
	spd.x+= (stats.speed * (1.0f - 0.5f*getState(CASTING|ATTACKING)));
	}

void Character::moveUp()
	{
	spd.y+= (stats.speed * (1.0f - 0.5f*getState(CASTING|ATTACKING)));
	}

void Character::moveDown()
	{
	spd.y+=-(stats.speed * (1.0f - 0.5f*getState(CASTING|ATTACKING)));
	}

void Character::jump(float force)
	{
	spd.z=force;

	pushEvent(Event::CHARACTER_JUMP, force);
	}
