/*
 * effects.h
 *
 *  Created on: 19-07-2013
 *      Author: crm
 */

#pragma once

#include "asmmath.h"

#include "ui/text.h"

//#include "main.h"
#include "effect.h"
#include "character.h"

namespace Effects
	{
	class Slide: public Effect
		{
		public:
			enum Type
				{
				LTR,	// lewo -> prawo
				RTL,	// prawo -> lewo
				TTB,	// gora -> dol
				BTT,	// dol -> gora
				};

			enum Function
				{
				LINEAR,
				EXPONENTIAL,
				};

		protected:
			Type type;
			Function fun;

			AVector startpos, endpos;

			void init();

		public:
			Slide(): Effect(), type(LTR), fun(LINEAR), startpos(0.0f, 0.0f, 0.0f, 0.0f), endpos(0.0f, 0.0f, 0.0f, 0.0f) {}

			virtual void init(const std::string& name, int timeout);
			virtual void update(float dt);
			virtual void print();

			void setType(Type stype) {type=stype; init();}
			void setFunction(Function sfun) {fun=sfun; init();}
			void setPos(AVector spos, AVector epos) {startpos=spos; endpos=epos;}
		};

	class Particle: public Effect
		{
		protected:
			AVector pos;
			AVector spd;
			AVector gravity;

		public:
			Particle(): Effect(), pos(0.0f, 0.0f, 0.0f, 0.0f), spd(0.0f, 0.0f, 0.0f, 0.0f), gravity(0.0f, 0.0f, -750.0f, 0.0f) {}

			virtual void update(float dt);
			virtual void print();

			//virtual bool isOver() const;	// Przerobic na inne?

			void setPos(AVector spos) {pos=spos;}
			void setSpd(AVector sspd) {spd=sspd;}
			void setSpd(float ang, float pow);
			void setGravity(AVector sg) {gravity=sg;}
		};

	class TextParticle: public Particle
		{
		protected:
			//UI::Text str;

		public:
			TextParticle(): Particle()/*, str()*/ {}

			virtual void init(const std::string& name, int timeout);
			virtual void print();
			virtual void clear();

			void setText(const std::string& sstr) {/*str=sstr; str.update();*/}
			void setFont(const std::string& font);// {str.setFont(font); str.init(); str.update();}
		};

	class SpeechBubble: public Effect
		{
		protected:
			Character *c;
			//Graphics::SpritePtr spr;

		public:
			SpeechBubble(): Effect()/*, c(NULL)*/ {}

			virtual void init(const std::string& name, int timeout);
			virtual void update(float dt);
			virtual void print();

			void setCharacter(Character *s) {c=s;}
			void setSprite(const std::string& name) {spr.setSprite(name);}
			void setAnim(unsigned sa) {spr.setAnim(sa);}
		};
	}
