/*
 * game.h
 *
 *  Created on: 02-03-2015
 *      Author: crm
 */

#pragma once

#include <vector>
#include <string>

#include "base/serializable.h"

class Arena;
class Gamemode;
class Character;
class Idea;

class Game: public Serializable
	{
	private:
		Arena *arena;
		Gamemode *gamemode;

		std::vector<Character *> characters;
		std::vector<Idea *> ideas;

		int nextCharacterID;
		int nextIdeaID;

	public:
		Game(): arena(NULL), gamemode(NULL), nextCharacterID(0), nextIdeaID(0) {}
		virtual ~Game() {clear();}

		bool init(Arena *arena, Gamemode *gamemode);
		void update(float dt);
		void print();
		void clear();

		Arena* getArena() const {return arena;}
		Gamemode* getGamemode() const {return gamemode;}

		int getCharacterCount() const {return characters.size();}
		Character* getCharacter(int index) const;
		Character* getCharacterByID(int id) const;

		int getIdeaCount() const {return ideas.size();}
		Idea* getIdea(int index) const;
		Idea* getIdeaByID(int id) const;

		void setArena(Arena *arena);
		void setGamemode(Gamemode *gamemode);

		Character* addCharacter(Character *c);
		Idea* addIdea(Idea *i);

		char* serialize(int &size);
		void deserialize(char *buff);
	};
