/*
 * Idea.h
 *
 *  Created on: 05-10-2014
 *      Author: crm
 */

#pragma once

#include "graphics/spriteptr.h"

#include "asmmath.h"

#include "base/serializable.h"
#include "base/eventprovider.h"

#include "objecttype.h"

class Character;
class GameMode;

class Idea: public Serializable, public ObjectType, public EventProvider
	{
	private:
		AVector pos;
		AVector spd;

		int id;
		static int nextid;

		Graphics::SpritePtr spr;

		Character* owner;
		GameMode* gamemode;

		float damage;
		float restitution;
		float cost;
		//float basespeed;

		struct IdeaSerialized
			{
			AVector pos;
			AVector spd;
			float damage;
			float restitution;
			};

	public:
		Idea(): pos(), spd(), id(++nextid), spr(), owner(NULL), gamemode(NULL), damage(0.0f), restitution(0.0f), cost(0.0f)/*, basespeed(100.0f)*/ {}
		virtual ~Idea() {}

		virtual bool init(const std::string& name, float damage, float cost, GameMode *arena, float restitution=0.5f, Character *parent=NULL);
		virtual void update(float dt);
		virtual void print();
		virtual void clear();

		virtual char* serialize(int &size);
		virtual void deserialize(char *buff);

		Idea* clone() const;

		//void pushEvent(unsigned e, float dataf=0.0f, int datai=0);
		//void pushEvent(const Event& e);

		inline AVector getPos() const {return pos;}
		inline AVector getSpd() const {return spd;}
		inline int getId() const {return id;}
		inline Character* getOwner() const {return owner;}
		inline GameMode* getGameMode() const {return gamemode;}
		inline float getDamage() const {return damage;}
		inline float getCost() const {return cost;}
		//inline float getBaseSpeed() const {return basespeed;}

		inline void setPos(AVector s) {pos=s;}
		inline void setSpd(AVector s) {spd=s;}
		inline void setId(int s) {id=s;}
		inline void setSprite(const std::string& name) {spr.setSprite(name);}
		inline void setOwner(Character *s) {owner=s;}
		inline void setGameMode(GameMode *s) {gamemode=s;}
		inline void setDamage(float s) {damage=s;}
		inline void setRestitution(float s) {restitution=s;}
		inline void setCost(float s) {cost=s;}
		//inline void setBaseSpeed(float s) {basespeed=s;}
	};

