/*
 * luafuncs.cpp
 *
 *  Created on: 28-04-2013
 *      Author: crm
 */

#include "luafuncs.h"

#include <cstdlib>
#include <cstring>

#ifdef WIN32
	#define WINDOWS
#else
	#ifdef WIN64
		#define WINDOWS
	#else
		#define LINUX
	#endif
#endif

#ifdef WINDOWS
	#include <lua/lua.hpp>
	#include <lua/lualib.h>
	#include <lua/lauxlib.h>
#else
	#include <lua.hpp>
	#include <lualib.h>
	#include <lauxlib.h>
#endif

#include "log.h"
#include "resource.h"
#include "lua2.h"
#include "lua_misc.h"

#include "sound.h"

#include "consts.h"
#include "base/event.h"
#include "base/gamemode.h"
#include "gameplay/objecttype.h"
#include "gameplay/arena.h"
#include "gameplay/character.h"
#include "gameplay/combomanager.h"

#define ASSERT_ARGS(args) \
	if(lua_gettop(L)<(args))\
		{\
		Log::fail("LuaFuncs.%s: Za malo argumentow (otrzymano %d, oczekiwano %d)", __FUNCTION__, lua_gettop(L), (args));\
		return 0;\
		}
#define ASSERT_CHARACTER(c) \
	if(c==NULL)\
		{\
		Log::fail("LuaFuncs.%s: NULL character", __FUNCTION__);\
		return 0;\
		}
#define ASSERT_ARENA(a) \
	if((a)==NULL)\
		{\
		Log::fail("LuaFuncs.%s: NULL arena", __FUNCTION__);\
		return 0;\
		}
#define ASSERT_GAMEMODE(g) \
	if((g)==NULL)\
		{\
		Log::fail("LuaFuncs.%s: NULL gamemode", __FUNCTION__);\
		return 0;\
		}


/**** Zwracanie zapisanych obiektow ****/
namespace LuaFuncs
	{
	//Level* getLvl(lua_State *L);
	//Object* getObj(unsigned int id);
	//Npc* getNpc(lua_State *L);

	int getRank(lua_State *L);
	Arena* getArena(lua_State *L);
	Character* getCharacter(lua_State *L);
	GameMode* getGameMode(lua_State *L);
	EventListener* getEventListener(lua_State *L);
	}

// http://lua-users.org/lists/lua-l/2004-11/msg00440.html // nested namespaces

/**** Deklaracje ****/
namespace LuaFuncs
	{
	struct Function
		{
		Function(unsigned p, int(*ptr)(lua_State*), const char* name):
			permissions(p), ptr(ptr), name(name) {}

		unsigned permissions;
		int(*ptr)(lua_State*);
		const char* name;
		};
	}

namespace LuaFuncs
	{
// Bajery
	int test(lua_State *L);
	int kill(lua_State *L);
	int getGlobalVal(lua_State *L);
	int getGlobalStr(lua_State *L);
	int setGlobalVal(lua_State *L);
	int setGlobalStr(lua_State *L);
	int include(lua_State *L);
	int getPath(lua_State *L);
	int dummy(lua_State *L);

// Arena
	namespace LArena
		{
		int setArena(lua_State *L);
		int setArenaSize(lua_State *L);
		int setArenaBg(lua_State *L);
		int getArenaSize(lua_State *L);
		int addArenaStartingPosition(lua_State *L);
		}

	namespace LCharacter
		{
	// Postacie graczy
		int setCharacterPosition(lua_State *L);
		int setCharacterSpeed(lua_State *L);
		int setCharacterDirection(lua_State *L);
		int setCharacterState(lua_State *L);
		//int setCharacterMaxHp(lua_State *L);
		//int setCharacterMaxMp(lua_State *L);
		//int setCharacterCastSpeed(lua_State *L);
		int setCharacterHp(lua_State *L);
		int setCharacterMp(lua_State *L);
		int setCharacterBlock(lua_State *L);
		int setCharacterSpeakBlock(lua_State *L);

		int characterSay(lua_State *L);
		int characterDefend(lua_State *L);
		int characterJump(lua_State *L);

		int getCharacterName(lua_State *L);
		int getCharacterMoveSpeed(lua_State *L);

		int getCharacterPosition(lua_State *L);
		int getCharacterSpeed(lua_State *L);
		int getCharacterDirection(lua_State *L);
		int getCharacterState(lua_State *L);
		int getCharacterMaxHp(lua_State *L);
		int getCharacterMaxMp(lua_State *L);
		int getCharacterCastSpeed(lua_State *L);
		int getCharacterHp(lua_State *L);
		int getCharacterMp(lua_State *L);
		int getCharacterBlock(lua_State *L);
		int getCharacterSpeakBlock(lua_State *L);

		int addCharacterSound(lua_State *L);
		int playCharacterSound(lua_State *L);
		}

// Postac gracza
	namespace LSelf
		{
		int setCharacter(lua_State *L);
		int setSprite(lua_State *L);
		int setShadow(lua_State *L);
		int setSpeech(lua_State *L);
		int setName(lua_State *L);
		int setMoveSpeed(lua_State *L);

		int setPosition(lua_State *L);
		int setSpeed(lua_State *L);
		int setDirection(lua_State *L);
		int setState(lua_State *L);
		int setMaxHp(lua_State *L);
		int setMaxMp(lua_State *L);
		int setCastSpeed(lua_State *L);
		int setHp(lua_State *L);
		int setMp(lua_State *L);
		int setBlock(lua_State *L);
		int setSpeakBlock(lua_State *L);

		int say(lua_State *L);
		int defend(lua_State *L);
		int jump(lua_State *L);

		int addCombo(lua_State *L);
		int endCombo(lua_State *L);
		int addIdea(lua_State *L);

		int getName(lua_State *L);
		int getMoveSpeed(lua_State *L);

		int getPosition(lua_State *L);
		int getSpeed(lua_State *L);
		int getDirection(lua_State *L);
		int getState(lua_State *L);
		int getMaxHp(lua_State *L);
		int getMaxMp(lua_State *L);
		int getCastSpeed(lua_State *L);
		int getHp(lua_State *L);
		int getMp(lua_State *L);
		int getBlock(lua_State *L);
		int getSpeakBlock(lua_State *L);
		int getID(lua_State *L);

		int addSound(lua_State *L);
		int playSound(lua_State *L);
		}
	}

namespace LuaFuncs
	{
	Function functions[]={
		//Function(NONE, test, "test"),
		Function(SINGLEPLAYER, kill, "kill"),
		Function(MULTIPLAYER_SERVER, getGlobalVal, "getGlobalVal"),
		Function(MULTIPLAYER_SERVER, getGlobalStr, "getGlobalStr"),
		Function(MULTIPLAYER_SERVER, setGlobalVal, "setGlobalVal"),
		Function(MULTIPLAYER_SERVER, setGlobalStr, "setGlobalStr"),
		Function(NONE, include, "include"),
		Function(INIT, getPath, "getPath"),

		Function(ARENA_INIT, LArena::setArena, "setArena"),
		Function(ARENA, LArena::setArenaSize, "setArenaSize"),
		Function(ARENA, LArena::setArenaBg, "setArenaBg"),
		Function(NONE, LArena::getArenaSize, "getArenaSize"),
		Function(ARENA_INIT, LArena::addArenaStartingPosition, "addArenaStartingPosition"),

		Function(CHARACTER, LCharacter::setCharacterPosition, "setCharacterPosition"),
		Function(CHARACTER, LCharacter::setCharacterSpeed, "setCharacterSpeed"),
		Function(CHARACTER, LCharacter::setCharacterDirection, "setCharacterDirection"),
		Function(CHARACTER, LCharacter::setCharacterState, "setCharacterState"),
		//Function(CHARACTER, LCharacter::setCharacterMaxHp, "setCharacterMaxHp"),
		//Function(CHARACTER, LCharacter::setCharacterMaxMp, "setCharacterMaxMp"),
		//Function(CHARACTER, LCharacter::setCharacterCastSpeed, "setCharacterCastSpeed"),
		Function(CHARACTER, LCharacter::setCharacterHp, "setCharacterHp"),
		Function(CHARACTER, LCharacter::setCharacterMp, "setCharacterMp"),
		Function(CHARACTER, LCharacter::setCharacterBlock, "setCharacterBlock"),
		Function(CHARACTER, LCharacter::setCharacterSpeakBlock, "setCharacterSpeakBlock"),
		Function(CHARACTER, LCharacter::characterSay, "characterSay"),
		Function(CHARACTER, LCharacter::characterDefend, "characterDefend"),
		Function(CHARACTER, LCharacter::characterJump, "characterJump"),
		Function(NONE, LCharacter::getCharacterName, "getCharacterName"),
		Function(NONE, LCharacter::getCharacterMoveSpeed, "getCharacterMoveSpeed"),
		Function(NONE, LCharacter::getCharacterPosition, "getCharacterPosition"),
		Function(NONE, LCharacter::getCharacterSpeed, "getCharacterSpeed"),
		Function(NONE, LCharacter::getCharacterDirection, "getCharacterDirection"),
		Function(NONE, LCharacter::getCharacterState, "getCharacterState"),
		Function(NONE, LCharacter::getCharacterMaxHp, "getCharacterMaxHp"),
		Function(NONE, LCharacter::getCharacterMaxMp, "getCharacterMaxMp"),
		Function(NONE, LCharacter::getCharacterCastSpeed, "getCharacterCastSpeed"),
		Function(NONE, LCharacter::getCharacterHp, "getCharacterHp"),
		Function(NONE, LCharacter::getCharacterMp, "getCharacterMp"),
		Function(NONE, LCharacter::getCharacterBlock, "getCharacterBlock"),
		Function(NONE, LCharacter::getCharacterSpeakBlock, "getCharacterSpeakBlock"),
		Function(CHARACTER, LCharacter::addCharacterSound, "addCharacterSound"),
		Function(CHARACTER, LCharacter::playCharacterSound, "playCharacterSound"),

		Function(CHARACTER_INIT, LSelf::setCharacter, "setCharacter"),
		Function(CHARACTER_INIT, LSelf::setSprite, "setSprite"),
		Function(CHARACTER_INIT, LSelf::setShadow, "setShadow"),
		Function(CHARACTER_INIT, LSelf::setSpeech, "setSpeech"),
		Function(CHARACTER_INIT, LSelf::setName, "setName"),
		Function(CHARACTER_INIT, LSelf::setMoveSpeed, "setMoveSpeed"),
		Function(CHARACTER_INIT, LSelf::setPosition, "setPosition"),
		Function(CHARACTER_INIT, LSelf::setSpeed, "setSpeed"),
		Function(CHARACTER_INIT, LSelf::setDirection, "setDirection"),
		Function(CHARACTER_INIT, LSelf::setState, "setState"),
		Function(CHARACTER_INIT, LSelf::setMaxHp, "setMaxHp"),
		Function(CHARACTER_INIT, LSelf::setMaxMp, "setMaxMp"),
		Function(CHARACTER_INIT, LSelf::setCastSpeed, "setCastSpeed"),
		Function(CHARACTER_INIT, LSelf::setHp, "setHp"),
		Function(CHARACTER_INIT, LSelf::setMp, "setMp"),
		Function(CHARACTER_INIT, LSelf::setBlock, "setBlock"),
		Function(CHARACTER_INIT, LSelf::setSpeakBlock, "setSpeakBlock"),
		Function(CHARACTER_SELF, LSelf::say, "say"),
		Function(CHARACTER_SELF, LSelf::defend, "defend"),
		Function(CHARACTER_SELF, LSelf::jump, "jump"),
		Function(CHARACTER_SELF, LSelf::addCombo, "addCombo"),
		Function(CHARACTER_SELF, LSelf::endCombo, "endCombo"),
		Function(CHARACTER_SELF, LSelf::addIdea, "addIdea"),
		Function(CHARACTER_SELF, LSelf::getName, "getName"),
		Function(CHARACTER_SELF, LSelf::getMoveSpeed, "getMoveSpeed"),
		Function(CHARACTER_SELF, LSelf::getPosition, "getPosition"),
		Function(CHARACTER_SELF, LSelf::getSpeed, "getSpeed"),
		Function(CHARACTER_SELF, LSelf::getDirection, "getDirection"),
		Function(CHARACTER_SELF, LSelf::getState, "getState"),
		Function(CHARACTER_SELF, LSelf::getMaxHp, "getMaxHp"),
		Function(CHARACTER_SELF, LSelf::getMaxMp, "getMaxMp"),
		Function(CHARACTER_SELF, LSelf::getCastSpeed, "getCastSpeed"),
		Function(CHARACTER_SELF, LSelf::getHp, "getHp"),
		Function(CHARACTER_SELF, LSelf::getMp, "getMp"),
		Function(CHARACTER_SELF, LSelf::getBlock, "getBlock"),
		Function(CHARACTER_SELF, LSelf::getSpeakBlock, "getSpeakBlock"),
		Function(CHARACTER_SELF, LSelf::getID, "getID"),
		Function(CHARACTER_SELF, LSelf::addSound, "addSound"),
		Function(CHARACTER_SELF, LSelf::playSound, "playSound"),
		};
	}

/**** Rejestrowanie ****/
namespace LuaFuncs
	{
	void load(Lua& lua, unsigned permissions)
		{
		lua.addGlobal("test", test);

		const unsigned size=sizeof(functions)/sizeof(functions[0]);
		for(unsigned i=0; i<size; ++i)
			{
			const Function &f=functions[i];

			if((permissions&f.permissions)==f.permissions)	// ma dostep
				lua.addGlobal(f.name, f.ptr);
			else
				lua.addGlobal(f.name, dummy);
			}
		}
	// int (.*)\(lua_State \*L\);			// <- Zamienic co
	// lua.addGlobal(LuaFuncs::\1, \"\1\");	// <- na co		// Regexp ftw


	int getRank(lua_State *L)
		{
		lua_pushstring(L, LUA_PTR_TYPE);
		lua_gettable(L, LUA_REGISTRYINDEX);

		int rank=lua_tointeger(L, -1);

		lua_settop(L, -2);

		return rank;
		}

	Arena* getArena(lua_State *L)
		{
		lua_pushstring(L, LUA_PTR_ARENA);
		lua_gettable(L, LUA_REGISTRYINDEX);

		Arena *a=static_cast<Arena *>(lua_touserdata(L, -1));

		lua_settop(L, -2);

		return a;
		}

	Character* getCharacter(lua_State *L)
		{
		lua_pushstring(L, LUA_PTR_CHARACTER);
		lua_gettable(L, LUA_REGISTRYINDEX);

		Character *c=static_cast<Character *>(lua_touserdata(L, -1));

		lua_settop(L, -2);

		return c;
		}

	GameMode* getGameMode(lua_State *L)
		{
		lua_pushstring(L, LUA_PTR_GAMEMODE);
		lua_gettable(L, LUA_REGISTRYINDEX);

		GameMode *c=static_cast<GameMode *>(lua_touserdata(L, -1));

		lua_settop(L, -2);

		return c;
		}

	EventListener* getEventListener(lua_State *L)
		{
		lua_pushstring(L, LUA_PTR_EVENT_LISTENER);
		lua_gettable(L, LUA_REGISTRYINDEX);

		EventListener *c=static_cast<EventListener *>(lua_touserdata(L, -1));

		lua_settop(L, -2);

		return c;
		}
	}

/**** Funkcje do dzialan specjalnych ****/
namespace LuaFuncs
	{
	int test(lua_State *L)
		{
		if(!L)
			return 0;
		printf("LuaFuncs.lTest");
		return 0;
		}

	int kill(lua_State *L)
		{
		if(!L)
			return 0;
		printf("INFO: LuaFuncs.%s: Zamykanie", __FUNCTION__);
		exit(0);
		}

	int getGlobalVal(lua_State *L)
		{
		if(!L)
			return 0;
		if(lua_gettop(L)<1)
			{
			printf("FAIL: LuaFuncs.%s: Za malo argumentow", __FUNCTION__);
			return 0;
			}
		lua_pushstring(L, lua_tostring(L, 1));
		lua_gettable(L, LUA_REGISTRYINDEX);
		lua_pushnumber(L, lua_tonumber(L, -1));
		return 1;
		}

	int getGlobalStr(lua_State *L)
		{
		if(!L)
			return 0;
		if(lua_gettop(L)<1)
			{
			printf("FAIL: LuaFuncs.%s: Za malo argumentow", __FUNCTION__);
			return 0;
			}
		lua_pushstring(L, lua_tostring(L, 1));
		lua_gettable(L, LUA_REGISTRYINDEX);
		lua_pushstring(L, lua_tostring(L, -1));
		return 1;
		}

	int setGlobalVal(lua_State *L)
		{
		if(!L)
			return 0;
		if(lua_gettop(L)<2)
			{
			printf("FAIL: LuaFuncs.%s: Za malo argumentow", __FUNCTION__);
			return 0;
			}
		const char* name=lua_tostring(L, 1);
		int val=lua_tonumber(L, 2);
		lua_pushstring(L, name);
		lua_pushnumber(L, val);
		lua_settable(L, LUA_REGISTRYINDEX);
		return 0;
		}

	int setGlobalStr(lua_State *L)
		{
		if(!L)
			return 0;
		if(lua_gettop(L)<2)
			{
			printf("FAIL: LuaFuncs.%s: Za malo argumentow", __FUNCTION__);
			return 0;
			}
		const char* name=lua_tostring(L, 1);
		const char* val=lua_tostring(L, 2);
		lua_pushstring(L, name);
		lua_pushstring(L, val);
		lua_settable(L, LUA_REGISTRYINDEX);
		return 0;
		}

	int include(lua_State *L)
		{
		if(!L)
			return 0;
		if(lua_gettop(L)<1)
			{
			printf("FAIL: LuaFuncs.%s: Za malo argumentow", __FUNCTION__);
			return 0;
			}

		char *script;

		if(getCharacter(L))
			script=Resource::load(getCharacter(L)->getPath()+":"+lua_tostring(L, 1));
		else
			script=Resource::load(lua_tostring(L, 1));//res.load(lua_tostring(L, 1));

		if(script==NULL)
			{
			printf("FAIL: LuaFuncs.%s: Nie znaleziono pliku \"%s\"", __FUNCTION__, lua_tostring(L, 1));
			return 0;
			}
		if(luaL_loadbufferx(L, script, Resource::getSize(), "include", NULL)!=LUA_OK)
			{
			printf("FAIL: LuaFuncs.%s: Problem przy ladowaniu skryptu; %s", __FUNCTION__, lua_tostring(L, -1));
			return 0;
			}
		if(lua_pcallk(L, 0, LUA_MULTRET, 0, 0, NULL)!=LUA_OK)
			{
			printf("FAIL: LuaFuncs.%s: Problem przy wykonaniu skryptu; %s", __FUNCTION__, lua_tostring(L, -1));
			return 0;
			}

		delete [] script;

		return 0;
		}

	int getPath(lua_State *L)
		{
		lua_pushstring(L, "__PATH__");
		lua_gettable(L, LUA_REGISTRYINDEX);
		//lua_pushstring(L, lua_tostring(L, -1));

		return 1;
		}

	int dummy(lua_State *)
		{
		return 0;
		}
	}

namespace LuaFuncs
	{
	/**** Arena ****/
	namespace LArena
		{
		int setArena(lua_State *L)
			{
			Arena *a=getArena(L);

			ASSERT_ARENA(a)
			ASSERT_ARGS(1)

			// x, y, w, h
			// bg
			// spectators
			// bgs front, bgs back -> x, y, sprite
			// startpos -> x, y

			lua_pushstring(L, "size"); lua_gettable(L, -2);
			if(!lua_isnil(L, -1))
				{
				int x, y, w, h;

				/*lua_pushnumber(L, 1); lua_gettable(L, -2); x=lua_tonumber(L, -1); lua_settop(L, -2);
				lua_pushnumber(L, 2); lua_gettable(L, -2); y=lua_tonumber(L, -1); lua_settop(L, -2);
				lua_pushnumber(L, 3); lua_gettable(L, -2); w=lua_tonumber(L, -1); lua_settop(L, -2);
				lua_pushnumber(L, 4); lua_gettable(L, -2); h=lua_tonumber(L, -1); lua_settop(L, -2);*/
				lua_pushstring(L, "x"); lua_gettable(L, -2); x=lua_tonumber(L, -1); lua_settop(L, -2);
				lua_pushstring(L, "y"); lua_gettable(L, -2); y=lua_tonumber(L, -1); lua_settop(L, -2);
				lua_pushstring(L, "w"); lua_gettable(L, -2); w=lua_tonumber(L, -1); lua_settop(L, -2);
				lua_pushstring(L, "h"); lua_gettable(L, -2); h=lua_tonumber(L, -1); lua_settop(L, -2);

				a->setSize(x, y, w, h);
				}
			lua_settop(L, -2);

			lua_pushstring(L, "background"); lua_gettable(L, -2);
			if(!lua_isnil(L, -1))
				if(getRank(L)!=ObjectType::OBJECT_HEADLESS)
					a->setBackground(lua_tostring(L, -1));
			lua_settop(L, -2);

			lua_pushstring(L, "object"); lua_gettable(L, -2);
			if(!lua_isnil(L, -1) && getRank(L)!=ObjectType::OBJECT_HEADLESS)
				{
				int i=1;

				while(true)
					{
					lua_pushnumber(L, i++);
					lua_gettable(L, -2);

					if(lua_isnil(L, -1))
						{
						lua_settop(L, -2);
						break;
						}

					float x, y, z;
					const char *name;

					/*lua_pushnumber(L, 1); lua_gettable(L, -2); x=lua_tonumber(L, -1); lua_settop(L, -2);
					lua_pushnumber(L, 2); lua_gettable(L, -2); y=lua_tonumber(L, -1); lua_settop(L, -2);
					lua_pushnumber(L, 3); lua_gettable(L, -2); z=lua_tonumber(L, -1); lua_settop(L, -2);
					lua_pushnumber(L, 4); lua_gettable(L, -2); name=lua_tostring(L, -1); lua_settop(L, -2);*/
					lua_pushstring(L, "x"); lua_gettable(L, -2); x=lua_tonumber(L, -1); lua_settop(L, -2);
					lua_pushstring(L, "y"); lua_gettable(L, -2); y=lua_tonumber(L, -1); lua_settop(L, -2);
					lua_pushstring(L, "z"); lua_gettable(L, -2); z=lua_tonumber(L, -1); lua_settop(L, -2);
					lua_pushstring(L, "name"); lua_gettable(L, -2); name=lua_tostring(L, -1); lua_settop(L, -2);

					a->addBackdrop( AVectorSet(x, y, z, 0.0f), name );

					lua_settop(L, -2);
					}
				}
			lua_settop(L, -2);

			/*lua_pushstring(L, "front"); lua_gettable(L, -2);
			if(!lua_isnil(L, -1) && getRank(L)!=ObjectType::OBJECT_HEADLESS)
				{
				int i=1;

				while(true)
					{
					lua_pushnumber(L, i++);
					lua_gettable(L, -2);

					if(lua_isnil(L, -1))
						{
						lua_settop(L, -2);
						break;
						}

					float x, y;
					const char *name;

					lua_pushnumber(L, 1); lua_gettable(L, -2); x=lua_tonumber(L, -1); lua_settop(L, -2);
					lua_pushnumber(L, 2); lua_gettable(L, -2); y=lua_tonumber(L, -1); lua_settop(L, -2);
					lua_pushnumber(L, 3); lua_gettable(L, -2); name=lua_tostring(L, -1); lua_settop(L, -2);

					a->addBackdrop( AVectorSet(x, y, 1.0f, 0.0f), name );

					lua_settop(L, -2);
					}
				}
			lua_settop(L, -2);*/

			lua_pushstring(L, "start"); lua_gettable(L, -2);
			if(!lua_isnil(L, -1))
				{
				int i=1;

				while(true)
					{
					lua_pushnumber(L, i++);
					lua_gettable(L, -2);

					if(lua_isnil(L, -1))
						break;

					float x, y, z;

					//luaStackDump(L);
					lua_pushnumber(L, 1); lua_gettable(L, -2); x=lua_tonumber(L, -1); lua_settop(L, -2);
					lua_pushnumber(L, 2); lua_gettable(L, -2); y=lua_tonumber(L, -1); lua_settop(L, -2);
					//lua_pushnumber(L, 3); lua_gettable(L, -2); z=(!lua_isnil(L, -1))?lua_tonumber(L, -1):0.0f; lua_settop(L, -2);
					lua_pushnumber(L, 3); lua_gettable(L, -2); z=lua_tonumber(L, -1); lua_settop(L, -2);

					a->addStartPos( AVectorSet(x, y, z, 0.0f) );

					lua_settop(L, -2);
					}
				lua_settop(L, -2);
				}
			lua_settop(L, -2);

			lua_pushstring(L, "camera"); lua_gettable(L, -2);
			if(!lua_isnil(L, -1))
				{
				float angle;
				float xmin, xmax;
				float y, z;

				lua_pushstring(L, "angle"); lua_gettable(L, -2); angle=lua_tonumber(L, -1); lua_settop(L, -2);
				lua_pushstring(L, "xmin"); lua_gettable(L, -2); xmin=lua_tonumber(L, -1); lua_settop(L, -2);
				lua_pushstring(L, "xmax"); lua_gettable(L, -2); xmax=lua_tonumber(L, -1); lua_settop(L, -2);
				lua_pushstring(L, "y"); lua_gettable(L, -2); y=lua_tonumber(L, -1); lua_settop(L, -2);
				lua_pushstring(L, "z"); lua_gettable(L, -2); z=lua_tonumber(L, -1); lua_settop(L, -2);

				if(xmin>xmax)
					{
					float tmp=xmin;
					xmin=xmax;
					xmax=tmp;
					}

				printf("%f %f %f %f %f", angle, xmin, xmax, y, z);

				a->getCamera()->look(AVector(0, 0, 0), AVector(0, -1, 0), AVector(0, 0, 1));
				a->getCamera()->setPos(AVector(xmin+(xmax-xmin)/2.0f, y, z));
				a->getCamera()->rotateRight(angle);

				}
			lua_settop(L, -2);

			return 0;
			}

		int setArenaSize(lua_State *L)
			{
			Arena *a=getArena(L);

			ASSERT_ARENA(a)
			ASSERT_ARGS(4)

			a->setSize(lua_tonumber(L, -4), lua_tonumber(L, -3), lua_tonumber(L, -2), lua_tonumber(L, -1));

			return 0;
			}

		int setArenaBg(lua_State *L)
			{
			Arena *a=getArena(L);

			ASSERT_ARENA(a)
			ASSERT_ARGS(1)

			if(getRank(L)!=ObjectType::OBJECT_HEADLESS)
				a->setBackground(lua_tostring(L, -1));

			return 0;
			}

		int getArenaSize(lua_State *L)
			{
			Arena *a=getArena(L);

			ASSERT_ARENA(a)

			// todo

			lua_pushnumber(L, 0.0f);
			lua_pushnumber(L, 0.0f);
			lua_pushnumber(L, 0.0f);
			lua_pushnumber(L, 0.0f);

			return 4;
			}

		int addArenaStartingPosition(lua_State *L)
			{
			Arena *a=getArena(L);

			ASSERT_ARENA(a)
			ASSERT_ARGS(1)

			if(getCharacter(L))
				{
				Log::fail("LuaFuncs.%s: Wywolanie tej funkcji spod skryptu postaci jest zabronione", __FUNCTION__);
				return 0;
				}

			return 0;
			}
		}

	/**** Character, all ****/
	namespace LCharacter
		{
		int setCharacterPosition(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(4)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			c->setPos( AVector(lua_tonumber(L, -3), lua_tonumber(L, -2), lua_tonumber(L, -1), 0.0f) );

			return 0;
			}

		int setCharacterSpeed(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(4)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			c->setSpd( AVector(lua_tonumber(L, -3), lua_tonumber(L, -2), lua_tonumber(L, -1)) );

			return 0;
			}

		int setCharacterDirection(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(2)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			if( strcmp(lua_tostring(L, -1), "left")==0 )
				c->setDir( Character::LEFT );
			else
				c->setDir( Character::RIGHT );

			return 0;
			}

		int setCharacterState(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(2)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			c->setState( lua_tointeger(L, -1) );	// Miejmy nadzieje, ze bedzie to dobry sposob na przeslanie maski bitowej

			return 0;
			}

		int setCharacterHp(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(2)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			c->setHp( lua_tonumber(L, -1) );

			return 0;
			}

		int setCharacterMp(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(2)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			c->setMp( lua_tonumber(L, -1) );

			return 0;
			}

		int setCharacterBlock(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(2)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			c->setBlock( lua_tonumber(L, -1) );

			return 0;
			}

		int setCharacterSpeakBlock(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(2)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			c->setCastBlock( lua_tonumber(L, -1) );

			return 0;
			}


		int characterSay(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(2)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			c->attack();

			return 0;
			}

		int characterDefend(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(1)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			c->defend();

			return 0;
			}

		int characterJump(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(2)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			c->jump(lua_tonumber(L, -1));

			return 0;
			}


		int lAddCharacterCombo(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(2)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			if(!lua_isfunction(L, -2))
				{
				Log::fail("LuaFuncs.%s: Pierwszy argument powinien byc funkcja (otrzymano \"%s\")", __FUNCTION__, lua_typename(L, lua_type(L, -2)));
				return 0;
				}

			if(!lua_istable(L, -1))
				{
				Log::fail("LuaFuncs.%s: Drugi argument powinien byc tablica (otrzymano \"%s\")", __FUNCTION__, lua_typename(L, lua_type(L, -2)));
				return 0;
				}

			ComboManager::Combo cmb;

			lua_pushvalue(L, -2);
			cmb.setRef(luaL_ref(L, LUA_REGISTRYINDEX));

			lua_insert(L, 1);	// Przesun tablice na poczatek stosu
			lua_settop(L, 1);	// Skasuj wszystko poza poczatkiem

			//Log::debug("LuaFuncs.%s: Stos: %d; Top: funkcja/tablica/nil: %d/%d/%d", __FUNCTION__, lua_gettop(L), lua_isfunction(L, -1), lua_istable(L, -1), lua_isnil(L, -1));

			lua_pushnil(L);		// Indeks 'zerowy' tablicy
			while(lua_next(L, -2))
				{
				//Log::debug("LuaFuncs.%s: %s - %s", __FUNCTION__, lua_typename(L, lua_type(L, -2)), lua_typename(L, lua_type(L, -1)));

				const char *EVENT=lua_tostring(L, -1);

					 if(strcmp(EVENT, "none")==0)		cmb.addEvent(Event::NONE);
				else if(strcmp(EVENT, "move")==0)		cmb.addEvent(Event::CHARACTER_MOVE);
				else if(strcmp(EVENT, "move-left")==0)	cmb.addEvent(Event::CHARACTER_MOVE_LEFT);
				else if(strcmp(EVENT, "move-right")==0)	cmb.addEvent(Event::CHARACTER_MOVE_RIGHT);
				else if(strcmp(EVENT, "move-up")==0)	cmb.addEvent(Event::CHARACTER_MOVE_UP);
				else if(strcmp(EVENT, "move-down")==0)	cmb.addEvent(Event::CHARACTER_MOVE_DOWN);
				else if(strcmp(EVENT, "charging")==0)	cmb.addEvent(Event::CHARACTER_CHARGING);
				else if(strcmp(EVENT, "got-idea")==0)	cmb.addEvent(Event::CHARACTER_GOT_IDEA);
				else if(strcmp(EVENT, "attack")==0)		cmb.addEvent(Event::CHARACTER_ATTACK);
				else if(strcmp(EVENT, "defend")==0)		cmb.addEvent(Event::CHARACTER_DEFEND);
				else if(strcmp(EVENT, "hit")==0)		cmb.addEvent(Event::CHARACTER_HIT);
				else if(strcmp(EVENT, "jump")==0)		cmb.addEvent(Event::CHARACTER_JUMP);
				else if(strcmp(EVENT, "combo")==0)		cmb.addEvent(Event::CHARACTER_COMBO);

				lua_pop(L, 1);
				}

			c->addCombo(cmb);
			//Log::debug("LuaFuncs.%s: Combo dodane (ref: %d)", __FUNCTION__, cmb.getRef());

			lua_settop(L, 0);

			return 0;
			}

		int lCharacterEndCombo(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(1)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			c->stopCombo();

			return 0;
			}


		int getCharacterName(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(1)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			lua_pushstring( L, c->getName().c_str() );

			return 1;
			}

		int getCharacterMoveSpeed(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(1)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			lua_pushnumber( L, c->getStats().speed );

			return 0;
			}

		int getCharacterPosition(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(1)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			lua_pushnumber( L, c->getPos().x );
			lua_pushnumber( L, c->getPos().y );
			lua_pushnumber( L, c->getPos().z );

			return 3;
			}

		int getCharacterSpeed(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(1)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			lua_pushnumber( L, c->getSpd().x );
			lua_pushnumber( L, c->getSpd().y );
			lua_pushnumber( L, c->getSpd().z );

			return 3;
			}

		int getCharacterDirection(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(1)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			if(c->getDir()==Character::LEFT)
				lua_pushstring( L, "left" );
			else
				lua_pushstring( L, "right" );

			return 1;
			}

		int getCharacterState(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(1)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			lua_pushnumber( L, c->getState() );

			return 1;
			}

		int getCharacterMaxHp(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(1)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			lua_pushnumber( L, c->getStats().maxHP );

			return 1;
			}

		int getCharacterHp(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(1)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			lua_pushnumber( L, c->getHp() );

			return 1;
			}

		int getCharacterMaxMp(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(1)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			lua_pushnumber( L, c->getStats().maxMP );

			return 1;
			}

		int getCharacterMp(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(1)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			lua_pushnumber( L, c->getMp() );

			return 1;
			}

		int getCharacterCastSpeed(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(1)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			lua_pushnumber( L, c->getStats().castspeed );

			return 1;
			}

		int getCharacterBlock(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(1)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			lua_pushnumber( L, c->getBlock() );

			return 1;
			}

		int getCharacterSpeakBlock(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(1)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			lua_pushnumber( L, c->getCastBlock() );

			return 1;
			}

		int addCharacterSound(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(2)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			lua_pushinteger(L, c->addSound(lua_tostring(L, -1)));

			return 1;
			}

		int playCharacterSound(lua_State *L)
			{
			GameMode *g;
			Character *c;

			ASSERT_ARGS(3)

			g=getGameMode(L);
			ASSERT_GAMEMODE(g);

			c=g->getCharacter(lua_tonumber(L, 1));
			ASSERT_CHARACTER(c);

			float vol=lua_tonumber(L, -1);
			const std::string& name=c->getSound(lua_tointeger(L, -2));

			if(getRank(L)!=ObjectType::OBJECT_HEADLESS)
				Sound::sound(name, vol);
			else
				{
				EventListener *l=getEventListener(L);

				if(!l)
					return 0;

				Event e=Event(Event::TYPE_SOUND, 0);
				e.sound.vol=vol;
				e.sound.path=new char[name.length()+1];
				strcpy(e.sound.path, name.c_str());

				l->pushEvent(e);
				}

			return 0;
			}
		}

	/**** Character, self ****/
	namespace LSelf
		{
		int setCharacter(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_ARGS(1);
			ASSERT_CHARACTER(c);

			lua_pushstring(L, "sprite"); lua_gettable(L, -2);
			if(!lua_isnil(L, -1))
				if(getRank(L)!=ObjectType::OBJECT_HEADLESS)
					c->setSprite( c->getPath()+":"+lua_tostring(L, -1) );
			lua_settop(L, -2);

			lua_pushstring(L, "shadow"); lua_gettable(L, -2);
			if(!lua_isnil(L, -1))
				if(getRank(L)!=ObjectType::OBJECT_HEADLESS)
					c->setSpriteShadow( c->getPath()+":"+lua_tostring(L, -1) );
			lua_settop(L, -2);

			lua_pushstring(L, "speech"); lua_gettable(L, -2);
			if(!lua_isnil(L, -1))
				//if(getRank(L)!=ObjectType::OBJECT_HEADLESS)	// patrz do setSpeech
					c->setSpriteSpeech( c->getPath()+":"+lua_tostring(L, -1) );
			lua_settop(L, -2);

			lua_pushstring(L, "name"); lua_gettable(L, -2);
			if(!lua_isnil(L, -1))
				c->setName(lua_tostring(L, -1));
			lua_settop(L, -2);

			lua_pushstring(L, "hp"); lua_gettable(L, -2);
			if(!lua_isnil(L, -1))
				c->setStatsMaxHp(lua_tonumber(L, -1));
			lua_settop(L, -2);

			lua_pushstring(L, "mp"); lua_gettable(L, -2);
			if(!lua_isnil(L, -1))
				c->setStatsMaxMp(lua_tonumber(L, -1));
			lua_settop(L, -2);

			lua_pushstring(L, "castSpeed"); lua_gettable(L, -2);
			if(!lua_isnil(L, -1))
				c->setStatsCastSpeed(lua_tonumber(L, -1));
			lua_settop(L, -2);

			lua_pushstring(L, "moveSpeed"); lua_gettable(L, -2);
			if(!lua_isnil(L, -1))
				c->setStatsMovementSpeed(lua_tonumber(L, -1));
			lua_settop(L, -2);

			return 0;
			}

		int setSprite(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_ARGS(1);
			ASSERT_CHARACTER(c);

			if(getRank(L)!=ObjectType::OBJECT_HEADLESS)
				c->setSprite( c->getPath()+":"+lua_tostring(L, -1) );

			return 0;
			}

		int setShadow(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_ARGS(1);
			ASSERT_CHARACTER(c);

			if(getRank(L)!=ObjectType::OBJECT_HEADLESS)
				c->setSpriteShadow( c->getPath()+":"+lua_tostring(L, -1) );

			return 0;
			}

		int setSpeech(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_ARGS(1);
			ASSERT_CHARACTER(c);

			//if(getRank(L)!=ObjectType::OBJECT_HEADLESS)	// nawet headless to potrzebuje, do przesylania eventu o efekcie
				c->setSpriteSpeech( c->getPath()+":"+lua_tostring(L, -1) );

			return 0;
			}

		int setName(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_ARGS(1);
			ASSERT_CHARACTER(c);

			c->setName( lua_tostring(L, -1) );

			return 0;
			}

		int setMoveSpeed(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_ARGS(1);
			ASSERT_CHARACTER(c);

			c->setStatsMovementSpeed( lua_tonumber(L, -1) );

			return 0;
			}

		int setMaxHp(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_ARGS(1);
			ASSERT_CHARACTER(c);

			c->setStatsMaxHp( lua_tonumber(L, -1) );

			return 0;
			}

		int setMaxMp(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_ARGS(1);
			ASSERT_CHARACTER(c);

			c->setStatsMaxMp( lua_tonumber(L, -1) );

			return 0;
			}

		int setCastSpeed(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_ARGS(1);
			ASSERT_CHARACTER(c);

			c->setStatsCastSpeed( lua_tonumber(L, -1) );

			return 0;
			}


		int setPosition(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_ARGS(1);
			ASSERT_CHARACTER(c);

			c->setPos( AVector(lua_tonumber(L, -3), lua_tonumber(L, -2), lua_tonumber(L, -1), 0.0f) );

			return 0;
			}

		int setSpeed(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_ARGS(1);
			ASSERT_CHARACTER(c);

			c->setSpd( AVector(lua_tonumber(L, -3), lua_tonumber(L, -2), lua_tonumber(L, -1)) );

			return 0;
			}

		int setDirection(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_ARGS(1);
			ASSERT_CHARACTER(c);

			if( strcmp(lua_tostring(L, -1), "left")==0 )
				c->setDir( Character::LEFT );
			else
				c->setDir( Character::RIGHT );

			return 0;
			}

		int setState(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_ARGS(1);
			ASSERT_CHARACTER(c);

			c->setState( lua_tointeger(L, -1) );

			return 0;
			}

		int setHp(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_ARGS(1);
			ASSERT_CHARACTER(c);

			c->setHp( lua_tonumber(L, -1) );

			return 0;
			}

		int setMp(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_ARGS(1);
			ASSERT_CHARACTER(c);

			c->setMp( lua_tonumber(L, -1) );

			return 0;
			}

		int setBlock(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_ARGS(1);
			ASSERT_CHARACTER(c);

			c->setBlock( lua_tonumber(L, -1) );

			return 0;
			}

		int setSpeakBlock(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_ARGS(1);
			ASSERT_CHARACTER(c);

			c->setCastBlock( lua_tonumber(L, -1) );

			return 0;
			}


		int say(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_ARGS(1);
			ASSERT_CHARACTER(c);

			c->attack();

			return 0;
			}

		int defend(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_CHARACTER(c);

			c->defend();

			return 0;
			}

		int jump(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_ARGS(1);
			ASSERT_CHARACTER(c);

			c->jump(lua_tonumber(L, -1));

			return 0;
			}


		int addCombo(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_ARGS(2);
			ASSERT_CHARACTER(c);

			if(!lua_isfunction(L, -2))
				{
				Log::fail("LuaFuncs.%s: Pierwszy argument powinien byc funkcja (otrzymano \"%s\")", __FUNCTION__, lua_typename(L, lua_type(L, -2)));
				return 0;
				}

			if(!lua_istable(L, -1))
				{
				Log::fail("LuaFuncs.%s: Drugi argument powinien byc tablica (otrzymano \"%s\")", __FUNCTION__, lua_typename(L, lua_type(L, -2)));
				return 0;
				}

			ComboManager::Combo cmb;

			lua_pushvalue(L, -2);
			cmb.setRef(luaL_ref(L, LUA_REGISTRYINDEX));

			lua_insert(L, 1);	// Przesun tablice na poczatek stosu
			lua_settop(L, 1);	// Skasuj wszystko poza poczatkiem

			//Log::debug("LuaFuncs.%s: Stos: %d; Top: funkcja/tablica/nil: %d/%d/%d", __FUNCTION__, lua_gettop(L), lua_isfunction(L, -1), lua_istable(L, -1), lua_isnil(L, -1));

			lua_pushnil(L);		// Indeks 'zerowy' tablicy
			while(lua_next(L, -2))
				{
				//Log::debug("LuaFuncs.%s: %s - %s", __FUNCTION__, lua_typename(L, lua_type(L, -2)), lua_typename(L, lua_type(L, -1)));

				const char *EVENT=lua_tostring(L, -1);

				     if(strcmp(EVENT, "move")==0)		cmb.addEvent(Event::CHARACTER_MOVE);
				else if(strcmp(EVENT, "move-left")==0)	cmb.addEvent(Event::CHARACTER_MOVE_LEFT);
				else if(strcmp(EVENT, "move-right")==0)	cmb.addEvent(Event::CHARACTER_MOVE_RIGHT);
				else if(strcmp(EVENT, "move-up")==0)	cmb.addEvent(Event::CHARACTER_MOVE_UP);
				else if(strcmp(EVENT, "move-down")==0)	cmb.addEvent(Event::CHARACTER_MOVE_DOWN);
				else if(strcmp(EVENT, "charging")==0)	cmb.addEvent(Event::CHARACTER_CHARGING);
				else if(strcmp(EVENT, "got-idea")==0)	cmb.addEvent(Event::CHARACTER_GOT_IDEA);
				else if(strcmp(EVENT, "attack")==0)		cmb.addEvent(Event::CHARACTER_ATTACK);
				else if(strcmp(EVENT, "defend")==0)		cmb.addEvent(Event::CHARACTER_DEFEND);
				else if(strcmp(EVENT, "hit")==0)		cmb.addEvent(Event::CHARACTER_HIT);
				else if(strcmp(EVENT, "jump")==0)		cmb.addEvent(Event::CHARACTER_JUMP);
				else if(strcmp(EVENT, "combo")==0)		cmb.addEvent(Event::CHARACTER_COMBO);

				lua_pop(L, 1);
				}

			c->addCombo(cmb);
			//Log::debug("LuaFuncs.%s: Combo dodane (ref: %d)", __FUNCTION__, cmb.getRef());

			lua_settop(L, 0);

			return 0;
			}

		int endCombo(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_CHARACTER(c);

			c->stopCombo();

			return 0;
			}

		int addIdea(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_CHARACTER(c);

			Idea *i;

			try
				{
				i=new Idea();
				}
			catch(std::bad_alloc& )
				{
				Log::fail("LuaFuncs.%s: Zabraklo pamieci", __FUNCTION__);
				return 0;
				}

			lua_pushstring(L, "sprite"); lua_gettable(L, -2);
			if(!lua_isnil(L, -1))
				i->setSprite( c->getPath()+":"+lua_tostring(L, -1) );
			lua_settop(L, -2);

			lua_pushstring(L, "damage"); lua_gettable(L, -2);
			if(!lua_isnil(L, -1))
				i->setDamage(lua_tonumber(L, -1));
			lua_settop(L, -2);

			lua_pushstring(L, "restitution"); lua_gettable(L, -2);
			if(!lua_isnil(L, -1))
				i->setRestitution(lua_tonumber(L, -1));
			lua_settop(L, -2);

			lua_pushstring(L, "cost"); lua_gettable(L, -2);
			if(!lua_isnil(L, -1))
				i->setCost(lua_tonumber(L, -1));
			lua_settop(L, -2);

			c->addIdea(i);

			return 0;
			}


		int getName(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_CHARACTER(c);

			lua_pushstring( L, c->getName().c_str() );

			return 1;
			}

		int getMoveSpeed(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_CHARACTER(c);

			lua_pushnumber( L, c->getStats().speed );

			return 0;
			}

		int getPosition(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_CHARACTER(c);

			lua_pushnumber( L, c->getPos().x );
			lua_pushnumber( L, c->getPos().y );
			lua_pushnumber( L, c->getPos().z );

			return 3;
			}

		int getSpeed(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_CHARACTER(c);

			lua_pushnumber( L, c->getSpd().x );
			lua_pushnumber( L, c->getSpd().y );
			lua_pushnumber( L, c->getSpd().z );

			return 3;
			}

		int getDirection(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_CHARACTER(c);

			if(c->getDir()==Character::LEFT)
				lua_pushstring( L, "left" );
			else
				lua_pushstring( L, "right" );

			return 1;
			}

		int getState(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_CHARACTER(c);

			lua_pushnumber( L, c->getState() );

			return 1;
			}

		int getMaxHp(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_CHARACTER(c);

			lua_pushnumber( L, c->getStats().maxHP );

			return 1;
			}

		int getHp(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_CHARACTER(c);

			lua_pushnumber( L, c->getHp() );

			return 1;
			}

		int getMaxMp(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_CHARACTER(c);

			lua_pushnumber( L, c->getStats().maxMP );

			return 1;
			}

		int getMp(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_CHARACTER(c);

			lua_pushnumber( L, c->getMp() );

			return 1;
			}

		int getCastSpeed(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_CHARACTER(c);

			lua_pushnumber( L, c->getStats().castspeed );

			return 1;
			}

		int getBlock(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_CHARACTER(c);

			lua_pushnumber( L, c->getBlock() );

			return 1;
			}

		int getSpeakBlock(lua_State *L)
			{
			Character *c=getCharacter(L);

			ASSERT_CHARACTER(c);

			lua_pushnumber( L, c->getCastBlock() );

			return 1;
			}

		int getID(lua_State *L)
			{
			GameMode *g;
			Character *c;

			g=getGameMode(L);
			ASSERT_ARENA(g);

			c=getCharacter(L);
			ASSERT_CHARACTER(c);

			const int size=g->getCharacterCount();		// todo a ID postaci to co jest? o:
			for(int i=0; i<size; ++i)
				{
				if(g->getCharacter(i)==c)
					{
					lua_pushnumber(L, i);
					return 1;
					}
				}

			return 0;
			}

		int addSound(lua_State *L)
			{
			Character *c;

			ASSERT_ARGS(1);

			c=getCharacter(L);
			ASSERT_CHARACTER(c);

			lua_pushinteger(L, c->addSound( c->getPath()+":"+lua_tostring(L, -1) ));

			return 1;
			}

		int playSound(lua_State *L)
			{
			Character *c;

			c=getCharacter(L);
			ASSERT_CHARACTER(c);

			if(getRank(L)==ObjectType::OBJECT_LOCAL)
				Sound::sound(c->getSound(lua_tointeger(L, -2)), lua_tonumber(L, -1));
			else
				{
				EventListener *l=getEventListener(L);

				if(!l)
					return 0;

				Event e=Event(Event::TYPE_CHARACTER, Event::CHARACTER_SOUND);
				e.character.who=c;
				e.character.dataf=lua_tonumber(L, -1);
				e.character.datai=lua_tointeger(L, -2);

				l->pushEvent(e);
				}

			return 0;
			}
		}
	}
