/*
 * menu.cpp
 *
 *  Created on: 02-08-2013
 *      Author: crm
 */

#include "menu.h"

#include "SDL/SDL.h"

#include "database/controllermanager.h"

/**** Option ****/
void Menu::Option::initValue(float smin, float smax, float schange)
	{
	min=smin;
	max=smax;
	change=schange;
	val=(min+max)/2;
	}
void Menu::Option::initSubLayer(Layer *s)
	{
	sub=s;
	}

void Menu::Option::increment()
	{
	if(type!=VALUE)
		return;

	val+=change;
	if(val>max)
		val=max;
	}
void Menu::Option::decrement()
	{
	if(type!=VALUE)
		return;

	val-=change;
	if(val<min)
		val=min;
	}

/**** Layer ****/
void Menu::Layer::clear()
	{
	for(std::vector<Option *>::iterator it=opts.begin(); it!=opts.end(); ++it)
		delete *it;
	opts.clear();
	}

void Menu::Layer::addOption(Option *opt)
	{
	opts.push_back(opt);
	}
void Menu::Layer::delOption(int i)
	{
	opts.erase(opts.begin()+i);
	if(i<opt)
		--opt;
	}

void Menu::Layer::setOption(int i)
	{
	opt=i;
	clamp();
	}

void Menu::Layer::prev()
	{
	--opt;
	if(opt<0)
		opt=(int)opts.size()-1;
	}
void Menu::Layer::next()
	{
	++opt;
	if(opt>(int)opts.size()-1)
		opt=0;
	}
void Menu::Layer::first()
	{
	opt=0;
	}
void Menu::Layer::last()
	{
	opt=opts.size()-1;
	}

void Menu::Layer::clamp()
	{
	if(opts.size()==0u)
		return;
	while(opt<0)
		opt+=opts.size();
	while(opt>=(int)opts.size())
		opt-=opts.size();
	}

/**** Menu ****/
void Menu::update(float dt)
	{
	Uint8 *keys=SDL_GetKeyState(NULL);

	if(keyblock>0.0f)
		keyblock-=dt;

	if(!ControllerManager::isPressedAny() && !keys[SDLK_RETURN] && !keys[SDLK_ESCAPE])
		keyblock=0.0f;

	if(keyblock>0.0f)
		;
	else if( ControllerManager::isPressed(Controller::UP) )
		{
		prev();
		keyblock=1.0f/4.0f;
		redraw=true;
		}
	else if( ControllerManager::isPressed(Controller::DOWN) )
		{
		next();
		keyblock=1.0f/4.0f;
		redraw=true;
		}
	else if( ControllerManager::isPressed(Controller::LEFT) )
		{
		decrement();
		//keyblock=1.0f/32.0f;
		keyblock=getOption()->getBlock();
		redraw=true;
		}
	else if( ControllerManager::isPressed(Controller::RIGHT) )
		{
		increment();
		//keyblock=1.0f/32.0f;
		keyblock=getOption()->getBlock();
		redraw=true;
		}
	else if( ControllerManager::isPressed(Controller::A) || keys[SDLK_RETURN])
		{
		down();
		keyblock=1.0f;
		redraw=true;
		}
	else if( ControllerManager::isPressed(Controller::B) || keys[SDLK_ESCAPE] )
		{
		up();
		keyblock=1.0f;
		redraw=true;
		}
	}

void Menu::clear()
	{
	blocked=false;
	exit=false;

	root.clear();

	stack.clear();
	}

Menu::Option* Menu::getOptionByTag(int tag)
	{
	std::vector<Layer *> todo;

	todo.push_back(&root);

	while(todo.size()>0u)
		{
		Layer *l=todo.back();
		todo.pop_back();

		for(int i=0; i<l->getOptionCount(); ++i)
			{
			if(l->getOption(i)->getTag()==tag)
				return l->getOption(i);
			if(l->getOption(i)->getSubLayer())
				todo.push_back(l->getOption(i)->getSubLayer());
			}
		}

	return NULL;
	}

void Menu::unlock()
	{
	blocked=false;
	}

void Menu::up()
	{
	if(isBlocked()) return;
	if(stack.size()>1)
		stack.pop_back();
	else
		{
		blocked=true;
		exit=true;
		}
	}
void Menu::down()
	{
	if(isBlocked()) return;
	Option *o=getLayer()->getOption();

	switch(o->getType())
		{
		case ACTION:
			blocked=true;
		break;

		case ENTER:
			if(o->getSubLayer())
				{
				Layer *l=o->getSubLayer();
				stack.push_back(l);
				l->setOption(0);
				}
		break;

		case RETURN:
			up();
		break;

		case VALUE:
			// todo
		break;
		}
	}
void Menu::prev()
	{
	if(isBlocked()) return;
	stack.back()->prev();
	}
void Menu::next()
	{
	if(isBlocked()) return;
	stack.back()->next();
	}
void Menu::first()
	{
	if(isBlocked()) return;
	stack.back()->first();
	}
void Menu::last()
	{
	if(isBlocked()) return;
	stack.back()->last();
	}
void Menu::increment()
	{
	if(isBlocked()) return;
	stack.back()->getOption()->increment();
	}
void Menu::decrement()
	{
	if(isBlocked()) return;
	stack.back()->getOption()->decrement();
	}

#include "log.h"
void Menu::print()
	{
	std::vector<Layer *> todo;

	todo.push_back(&root);

	while(todo.size()>0u)
		{
		Layer *l=todo.back();
		todo.pop_back();

		for(int i=0; i<l->getOptionCount(); ++i)
			{
			if(l->getOption(i)->getSubLayer())
				todo.push_back(l->getOption(i)->getSubLayer());

			bool curr=l->getOption()==l->getOption(i);

			printf("%d: %s [%s][%s]", i, l->getOption(i)->getName().c_str(), (l->getOption(i)->getSubLayer())?"sub":"", (curr)?"curr":"");
			}
		}
	}
