/*
 * clientdialogi.h
 *
 *  Created on: 21-08-2013
 *      Author: crm
 */

#pragma once

#include <string>

#include "../player.h"
#include "../sockclient.h"
#include "../transferrer.h"
#include "base/controller.h"
#include "controllers/dummy.h"
//#include "../../bot.h"

class GameMode;
class Arena;

namespace Network
	{
	class Client
		{
		public:
			enum ClientState
				{
				NOT_CONNECTED,
				NEGOTIATION,
				READY,
				};

		private:
			SocketClient client;

			ClientState state;
			float delay;

			GameMode *gamemode;
			Arena *arena;
			std::vector<Character *> chars;

			Player self;
			Controller *ctrl;
			int ctrlState;

			Transferrer transfer;

			std::string addr;
			int port;
			int reconnects;

			void send(int type, int length, const char *data);

			void updateControls(float dt);
			void parse(Packet *pack);

			void parseWelcomeResponse(Packet *pack);
			void parseInfoCharacter(Packet *pack);
			void parseInfoServer(Packet *pack);
			void parseInfoArena(Packet *pack);
			void parseCharacterAdd(Packet *pack);
			void parseCharacterDel(Packet *pack);
			void parseStateCharacter(Packet *pack);
			//void parseStateArena(Packet *pack);
			void parseStateGamemode(Packet *pack);
			void parseEventCharacter(Packet *pack);
			void parseEventGamemode(Packet *pack);
			void parseMiscPlaySound(Packet *pack);
			void parseMiscPlayMusic(Packet *pack);
			void parseRequest(Packet *pack);
			void parseRequestCharacter(Packet *pack);
			void parseDataInfo(Packet *pack);
			void parseData(Packet *pack);

			void sendWelcome();
			void sendReady();
			void sendInfoCharacter();
			void sendInfoCharacter(int cid);
			void sendEventController();
			void sendRequestCharacter(unsigned crc);
			void sendRequestArena(unsigned crc);

			void addCharacter(Character *c);
			Character* getCharacter(int cid);

		public:
			 Client(): client(), state(NOT_CONNECTED), delay(0.0f), gamemode(NULL), arena(NULL),
			 	 self(), ctrl(NULL), ctrlState(0), addr(), port(0), reconnects(0) {}
			~Client() {clear();}

			bool init(const std::string& address, int port);
			void update(float dt);
			void print();
			void clear();

			bool connect();
			void disconnect();

			void setController(Controller *s);
			void setCharacter(Character *c);

			//Controller* getController(int i) const;// {return ctrls[i];}
			//Character* getCharacter(int i) const;// {return chars[i];}
			GameMode* getGameMode() const {return gamemode;}
			Arena* getArena() const {return arena;}
			int getCharacterCount() const {return chars.size();}
			//int getSelfId() const {return self.id;}
			int getReconnectCount() const {return reconnects;}
		};
	}
