/*
 * main.h
 *
 *  Created on: 04-09-2013
 *      Author: crm
 */

#pragma once

#include <string>
#include <vector>

#include "../sockserver.h"
#include "../player.h"
#include "../packet_dialogi.h"
#include "../transferrer.h"
#include "base/eventlistener.h"
#include "base/gamemode.h"
#include "controllers/dummy.h"
#include "gameplay/arena.h"
#include "gameplay/character.h"

namespace Network
	{
	const int SERVER_DEFAULT_PORT=16700;
	const int PLAYER_TIME_TO_LIVE=30*1000;		// Czas po jakim zostanie skasowany gracz ktory sie rozlaczyl (inaczej niz przez pakiet pozegnalny); 30*1000ms
	const int PLAYER_TIME_TO_STATE=2000;
	const int PLAYER_TIME_TO_STATE_RANDOM=500;

	class Server: public EventListener
		{
		private:
			struct ServerPlayer: public Player
				{
				enum ClientState
					{
					NOT_CONNECTED=			0x0,
					CONNECTED=				0x1<<0,
					HAS_CHARACTER=			0x1<<1,

					CAN_RECEIVE_BROADCAST=	CONNECTED,
					};

				ServerPlayer(): state(NOT_CONNECTED), timetolive(0), timetostate(0), socketid(-1) {}

				inline void setState(int s) {state=s;}
				inline void addState(int s) {state|=s;}
				inline void delState(int s) {state&=~s;}
				inline bool getState(int s) {return (state&s)==s;}

				int state;
				int timetolive;
				int timetostate;	// Czas do kolejnego wyslania stanow postaci i areny
				int socketid;

				Transferrer transfer;
				};

			int port;
			int maxclients;

			SocketServer server;

			std::vector<ServerPlayer *> players;
			std::vector<Character *> chars;
			//std::vector<Idea *> ideas;
			GameMode *gamemode;
			Arena *arena;

			int currentPid;
			float updateDelay;
			bool gameModeState;

			void parse(int socket, Packet *pack);
			void send(int socket, int type, int length, const char *data);
			void broadcast(int type, int length, const char *data);

			void updatePlayers(float dt);
			void updateGameplay(float dt);
			void updateEvents(float dt);

			ServerPlayer* findPlayerByID(int pid);
			ServerPlayer* findPlayerBySocketID(int socket);

			void disconnect(ServerPlayer *sp);

			void parseWelcome(int socket, Packet *pack);
			void parseBye(int socket, Packet *pack);
			void parseInfoCharacter(int socket, Packet *pack);
			void parseInfoServer(int socket, Packet *pack);
			void parseInfoArena(int socket, Packet *pack);
			void parseEventController(int socket, Packet *pack);
			void parseRequest(int socket, Packet *pack);

			void sendWelcomeResponse(int socket, int pid);
			void sendBye(int socket);
			void sendInfoCharacter(Character *c=NULL);
			void sendInfoCharacter(int socket, Character *c=NULL);
			void sendInfoServer(int socket);
			void sendInfoArena(int socket);
			void sendCharacterAdd(Character *c);
			void sendCharacterDel(Character *c);
			void sendCharacterAdd(int socket, Character *c);
			void sendCharacterDel(int socket, Character *c);
			void sendStateCharacter(int socket, Character *c=NULL);
			//void sendStateArena(int socket);
			void sendStateGamemode(int socket);
			void sendEventCharacter(Character *c, const Event& e);
			void sendRequestCharacter(int socket, unsigned crc);

			void addCharacter(Character *c);
			Character* getCharacter(int cid);

		public:
			Server(): server(), gamemode(NULL), arena(NULL), updateDelay(0.0f) {}
			virtual ~Server() {clear();}

			bool init(GameMode *gamemode, Arena *arena, int port=SERVER_DEFAULT_PORT);
			void update(float dt);
			void print();
			void clear();

			void setGamemode(GameMode *gamemode);
			void setArena(Arena *arena);

			int getPort() const {return port;}
			//int getMaxClientCount() const {return maxclients;}
		};
	}
