/*
 * client.cpp
 *
 *  Created on: 19-08-2013
 *      Author: crm
 */

#include "sockclient.h"

#include <ctime>

#include "log.h"

#include "clock.h"

using namespace Network;

const unsigned PACKET_HEADER=Packet::SIZE;
const unsigned PACKET_TIMEOUT=5000;		// Ping co 5 sekund
const unsigned PACKET_MAX_TIMEOUTS=5;
const unsigned PACKET_MAX_SEND_TIME=5;	// Max 5ms na wyslanie pakietu

//int time();// {return clock()*1000/CLOCKS_PER_SEC;}

bool SocketClient::init(const std::string& address, int port)
	{
	IPaddress ip;

	clear();

	SDLNet_ResolveHost(&ip, address.c_str(), port);
	if(ip.host==INADDR_NONE)
		{
		Log::fail("SocketClient.init: Nie udalo sie znalezc adresu \"%s\"", address.c_str());
		return false;
		}

	sock=SDLNet_TCP_Open(&ip);
	if(!sock)
		{
		Log::fail("SocketClient.init: Nie udalo sie otworzyc socketa (adres \"%s\" port %d)", address.c_str(), port);
		// Tu nie ma jeszcze czego kasowac, wiec clear nie jest potrzebny
		return false;
		}

	sset=SDLNet_AllocSocketSet(1);
	if(!sset)
		{
		Log::fail("SocketClient.init: Nie udalo sie stworzyc socket seta");
		clear();
		return false;
		}

	if(SDLNet_TCP_AddSocket(sset, sock)<1)
		{
		Log::fail("SocketClient.init: Nie udalo sie dodac socketa do socket seta");
		clear();
		return false;
		}

	lag=0u;
	ping=Clock::getTimeMilis();
	timeouts=0;
	recvd=0;
	pack=NULL;

	running=true;

	mutex=SDL_CreateMutex();
	if(!mutex)
		{
		Log::fail("SocketServer.init: Nie udalo sie stworzyc mutexa");
		clear();
		return false;
		}

	thread=SDL_CreateThread(run, this);
	if(!thread)
		{
		Log::fail("SocketServer.init: Nie udalo sie wystartowac watku");
		clear();
		return false;
		}

	Log::ok("SocketClient.init: Polaczono z %s:%d", address.c_str(), port);

	return true;
	}

bool SocketClient::init(TCPsocket ssock)
	{
	clear();

	if(!ssock)
		{
		Log::fail("SocketClient.init(TCP_SOCK): NULL socket");
		return false;
		}

	sock=ssock;

	sset=SDLNet_AllocSocketSet(1);
	if(!sset)
		{
		Log::fail("SocketClient.init(TCP_SOCK): Nie udalo sie stworzyc socket seta");
		clear();
		return false;
		}

	if(SDLNet_TCP_AddSocket(sset, sock)<1)
		{
		Log::fail("SocketClient.init(TCP_SOCK): Nie udalo sie dodac socketa do socket seta");
		clear();
		return false;
		}

	lag=0u;
	ping=Clock::getTimeMilis();
	timeouts=0;
	recvd=0;
	pack=NULL;

	running=true;

	/**** Do wypisania informacji o polaczeniu ****/
	IPaddress *ip;

	ip=SDLNet_TCP_GetPeerAddress(sock);

	if(!ip)
		{
		unlock();
		Log::info("SocketClient.init(TCP_SOCK): Nie udalo sie pobrac adresu socketa, czy ktos w ogole jest polaczony?");
		return true;
		}

	const char *address=SDLNet_ResolveIP(ip);

	if(address)
		Log::ok("SocketClient.init(TCP_SOCK): Polaczono z %s:%hu", address, ip->port);
	else
		Log::ok("SocketClient.init(TCP_SOCK): Polaczono z %d.%d.%d.%d:%hd", (ip->host>>0)&0xFF, (ip->host>>8)&0xFF, (ip->host>>16)&0xFF, (ip->host>>24)&0xFF, ip->port);
	/**** Koniec ****/

	mutex=SDL_CreateMutex();
	if(!mutex)
		{
		Log::fail("SocketServer.init(TCP_SOCK): Nie udalo sie stworzyc mutexa");
		clear();
		return false;
		}

	thread=SDL_CreateThread(run, this);
	if(!thread)
		{
		Log::fail("SocketServer.init(TCP_SOCK): Nie udalo sie wystartowac watku");
		clear();
		return false;
		}

	return true;
	}

void SocketClient::disconnect()
	{
	lock();

	if(running)
		{
		running=false;

		Log::info("SocketClient.disconnect: Rozlaczono");
		}

	unlock();
	}

void SocketClient::clear()
	{
	if(mutex)
		disconnect();

	if(thread)
		SDL_WaitThread(thread, NULL);
	thread=NULL;

	if(mutex)
		SDL_DestroyMutex(mutex);
	mutex=NULL;

	if(sock)
		SDLNet_TCP_Close(sock);
	if(sset)
		SDLNet_FreeSocketSet(sset);

	for(std::list<Packet *>::iterator it=inbuff.begin(); it!=inbuff.end(); ++it)
		delete [] *it;
	for(std::list<Packet *>::iterator it=outbuff.begin(); it!=outbuff.end(); ++it)
		delete [] *it;
	if(pack)
		delete [] pack;

	sock=NULL;
	sset=NULL;
	pack=NULL;

	inbuff.clear();
	outbuff.clear();
	buff.clear();
	}


void SocketClient::update()
	{
	updateRecv();
	updateSend();
	updatePing();
	}

void SocketClient::updateRecv()
	{
	int tmp_recvd;
	int ready;

	ready=SDLNet_CheckSockets(sset, 1);
	if(ready<1)
		return;

	// Nie ma potrzeby sprawdzac czy socket jest ready - jest jedynym w ssecie
	//if(!SDLNet_SocketReady(sock))
	//	return;

	//ping=time();
	//timeouts=0u;	// Niby zyje, ale nie wiem jak powinienem traktowac jego ignorowanie mnie :F

	if(!pack)
		{
		if(recvd==0u)
			{
			buff.resize(PACKET_HEADER+32);	// Wielkosc pakietu + 32 zapasowe bajty
			tmp_recvd=SDLNet_TCP_Recv(sock, buff.data(), PACKET_HEADER);

			if(tmp_recvd<0)
				{
				disconnect();
				return;
				}

			recvd+=tmp_recvd;
			addReadSum(tmp_recvd);

			if(recvd<PACKET_HEADER)
				return;
			}
		if(recvd<PACKET_HEADER)
			{
			buff.resize(PACKET_HEADER);
			tmp_recvd=SDLNet_TCP_Recv(sock, buff.data()+recvd, PACKET_HEADER-recvd);

			if(tmp_recvd<0)
				{
				disconnect();
				return;
				}

			recvd+=tmp_recvd;
			addReadSum(tmp_recvd);

			if(recvd<PACKET_HEADER)
				return;
			}
		if(recvd==PACKET_HEADER)
			{
			Packet *tmp_pack;
			int type, length/*, time*/;

			tmp_pack=(Packet *)buff.data();

			type=tmp_pack->type;
			length=tmp_pack->length;
			//time=tmp_pack->time;

			if(length>4096)
				{
				Log::fail("SocketClient.updateRecv: Duzy pakiet (typ %d wielkosc %d)", type, length);
				// Olac czy cos zrobic~?
				}

			pack=(Packet *)new unsigned char[PACKET_HEADER+length];

			pack->type=type;
			pack->length=length;
			//pack->time=time;

			recvd=0;
			//pack=tmp_pack;
			buff.resize(pack->length+1);
			//packets.push_back(pack);

			//Log::info("SocketClient.updateClient: Pobieranie pakietu; typ %d wielkosc %d", pack->type, pack->length);
			}
		}

	if(pack)
		{
		if(recvd<pack->length)
			{
			tmp_recvd=SDLNet_TCP_Recv(sock, buff.data()+recvd, pack->length-recvd);

			recvd+=tmp_recvd;
			addReadSum(tmp_recvd);

			if(tmp_recvd<0)
				{
				disconnect();
				return;
				}
			}

		if(recvd>=pack->length)
			{
			for(unsigned i=0u; i<pack->length; ++i)
				pack->data[i]=buff[i];

			//Log::info("SocketClient.updateRecv: Otrzymano pakiet; typ %d wielkosc %d", pack->type, pack->length);

			//if(Clock::getTimeMilis()-pack->time>20)
			//	Log::info("client lag %d, pakiet %d %d", Clock::getTimeMilis()-pack->time, pack->type, pack->length);

			if(pack->type<=0)
				{
				parsePing(pack);
				delete [] pack;
				}
			else
				{
				lock();
				inbuff.push_back(pack);
				unlock();
				}

			recvd=0u;
			pack=NULL;
			}
		}
	}

void SocketClient::updateSend()
	{
	unsigned sum=0u;

	lock();
	if(outbuff.size()<1)
		{
		unlock();
		return;
		}

	/*while(outbuff.size()>10)
		{
		Log::info("SocketClient.updateSend: Za duzo pakietow do wyslania, kasowanie");
		outbuff.pop_front();
		}*/

	while(outbuff.size()>0u)
	//for(std::list<Packet*>::iterator it=outbuff.begin(); it!=outbuff.end(); ++it)
		{
		const Packet *pack=outbuff.front();
		//const Packet *pack=*it;

		const unsigned send=Packet::SIZE+pack->length;

		outbuff.pop_front();

		const unsigned sent=SDLNet_TCP_Send(sock, pack, send);

		delete [] pack;

		if(sent!=send)
			{
			outbuff.clear();
			unlock();

			Log::info("SocketClient.updateSend: Rozlaczanie (wyslano %u, wysylano%u)", sent, send);

			disconnect();
			return;
			}

		sum+=sent;
		}

	//outbuff.clear();

	unlock();

	addSendSum(sum);
	}

void SocketClient::updatePing()
	{
	const unsigned TIME=Clock::getTimeMilis();

	if(timeouts>PACKET_MAX_TIMEOUTS)
		{
		disconnect();
		return;
		}

	if(ping+PACKET_TIMEOUT<TIME)
		{
		send(-1, sizeof(unsigned), (const char *)&TIME);
		ping=TIME;
		++timeouts;

		//Log::info("SocketClient.updatePing: Wysylanie pinga, czas %u, zignorowanych: %u", ping, timeouts);
		}
	}


void SocketClient::parsePing(Packet *pack)
	{
	if(pack->type==0)
		{
		send(pack->type, pack->length, pack->data);
		}
	else
		{
		const int t=*(int *)pack->data;

		timeouts=0;

		lock();
		lag=Clock::getTimeMilis()-t;
		unlock();

		//Log::debug("SocketClient.parsePing: Lag %u", lag);
		}
	}


void SocketClient::send(int type, int length, const char *data)
	{
	lock();
	if(!sock)
		{
		unlock();
		return;
		}

	if(!data)
		length=0;

	Packet *pack=(Packet*)new char[Packet::SIZE+length];

	pack->type=type;
	pack->length=length;
	//pack->time=Clock::getTimeMilis();

	if(data)
		memcpy(pack->data, data, length);

	outbuff.push_back(pack);

	unlock();
	}


int SocketClient::run(void *data)
	{
	//Log::info("SocketClient.run: Pamietaj, ze dziala na 1 watku");	// I, w razie czego, zakomentuj update w Client.update
	//return 0;

	if(!data)
		return 1;

	//Log::info("SocketClient.run: Watek wystartowal");

	SocketClient *c=(SocketClient*)data;

	while(c->isOk())
		{
		c->update();
		}

	c->updateSend();

	if(c->sock)
		SDLNet_TCP_Close(c->sock);
	if(c->sset)
		SDLNet_FreeSocketSet(c->sset);
	c->sock=NULL;
	c->sset=NULL;

	//Log::debug("SocketClient.run: Watek zatrzymany");

	return 0;
	}
