/*
 * client.h
 *
 *  Created on: 19-08-2013
 *      Author: crm
 */

#pragma once

extern "C"
	{
	#include <SDL/SDL_net.h>
	#include <SDL/SDL_thread.h>
	}

#include <list>
#include <vector>
#include <string>

#include "packet.h"

namespace Network
	{
	class SocketClient
		{
		/*public:
			enum ThreadState
				{
				RUNNING,
				STOPPING,
				STOPPED,
				};*/

		private:
			TCPsocket sock;
			SDLNet_SocketSet sset;

			unsigned lag;		// Czas
			unsigned ping;		// Czas ostatniego pinga (unix timestamp)
			unsigned timeouts;	// Liczba niedostarczonych pingow od ostatniego udanego
			unsigned recvd;		// Pobrane dane

			std::vector<unsigned char> buff;
			std::list<Packet *> inbuff;
			std::list<Packet *> outbuff;
			Packet *pack;

			int id;
			bool running;
			bool dnd;
			SDL_Thread *thread;
			SDL_mutex *mutex;
			unsigned send_sum;
			unsigned read_sum;

			static int run(void *data);

			void update();
			void updateRecv();
			void updateSend();
			void updatePing();
			void parsePing(Packet *pack);

			void   lock() const {SDL_mutexP(mutex);}
			void unlock() const {SDL_mutexV(mutex);}

			void addSendSum(unsigned s) {lock(); send_sum+=s; unlock();}
			void addReadSum(unsigned s) {lock(); read_sum+=s; unlock();}

		public:
			SocketClient(): sock(NULL), sset(NULL), lag(0u), ping(0u), timeouts(0u), recvd(0u), pack(NULL), id(-1), running(false), dnd(false), thread(NULL), mutex(NULL), send_sum(0), read_sum(0) {}
			~SocketClient() {clear();}

			bool init(const std::string& address, int port);
			bool init(TCPsocket sock);
			//void update();
			void disconnect();
			void clear();

			void send(int type, int length, const char *data);

			void setId(int s) {id=s;}
			int getId() const {return id;}

			void setDNDon() {lock(); dnd=true; unlock();}
			void setDNDoff() {lock(); dnd=false; unlock();}

			bool isOk() const			{    bool r; lock(); r=running;          unlock(); return r;}
			bool getDND() const			{    bool r; lock(); r=dnd;              unlock(); return r;}
			bool hasPacket() const		{    bool r; lock(); r=inbuff.size()>0u; unlock(); return r;}
			unsigned getLag() const		{unsigned r; lock(); r=lag;              unlock(); return r;}
			unsigned getSendSum() {unsigned r; lock(); r=send_sum; send_sum=0u; unlock(); return r;}
			unsigned getReadSum() {unsigned r; lock(); r=read_sum; read_sum=0u; unlock(); return r;}

			Packet* getPacket() {Packet *p; lock(); p=inbuff.front(); inbuff.pop_front(); unlock(); return p;}
		};
	}
