/*
 * server.cpp
 *
 *  Created on: 17-08-2013
 *      Author: crm
 */

#include "sockserver.h"

#include <ctime>

#include "log.h"

#include "clock.h"

using namespace Network;

bool SocketServer::init(int port, int maxclients)
	{
	IPaddress ip;
	SDLNet_ResolveHost(&ip, NULL, port);
	if(ip.host==INADDR_NONE)
		{
		Log::fail("SocketServer.init: Nie udalo sie znalezc hosta lokalnego, lol");
		return false;
		}

	server=SDLNet_TCP_Open(&ip);
	if(!server)
		{
		Log::fail("SocketServer.init: Nie udalo sie otworzyc socketa (port %d)", port);
		return false;
		}

	sset=SDLNet_AllocSocketSet(1); // +1 -> serwer
	if(!sset)
		{
		Log::fail("SocketServer.init: Nie udalo sie stworzyc socket seta");
		return false;
		}

	if(SDLNet_TCP_AddSocket(sset, server)<1)
		{
		Log::fail("SocketServer.init: Nie udalo sie dodac serwera do socket seta");
		return false;
		}

	clients.reserve(maxclients);

	//client_count=0;

	/*running=true;
	mutex=SDL_CreateMutex();
	if(!mutex)
		{
		Log::fail("SocketServer.init: Nie udalo sie stworzyc mutexa");
		return false;
		}

	thread=SDL_CreateThread(run, this);
	if(!thread)
		{
		Log::fail("SocketServer.init: Nie udalo sie wystartowac watku");
		return false;
		}*/

	Log::ok("SocketServer.init: Serwer wlaczony na porcie %d", port);
	Log::debug("SocketServer.init: Pamietaj, ze nie idzie na watkach");

	return true;
	}

void SocketServer::disconnect(int id)
	{
	if(id<0 || id>=getClientCount())
		return;

	SocketClient *c=getClient(id);
	if(c)
		c->disconnect();
	}

void SocketServer::clear()
	{
	SocketClient *c;
	for(unsigned i=0u; i<clients.size(); ++i)
	//for(std::list<SocketClient*>::iterator it=clients.begin(); it!=clients.end(); ++it)
		{
		c=clients[i];//*it;

		if(!c)
			continue;

		c->clear();
		delete c;
		}

	clients.clear();

	if(server)
		SDLNet_TCP_Close(server);
	if(sset)
		SDLNet_FreeSocketSet(sset);

	server=NULL;
	sset=NULL;

	eventListeners.clear();
	}


void SocketServer::update()
	{
	// Klientowe sockety update'uja sie same (watki~)
	updateJoin();
	updateClients();
	}

void SocketServer::updateJoin()
	{
	SDLNet_CheckSockets(sset, 0);
	if(!SDLNet_SocketReady(server))
		return;

	TCPsocket client;
	IPaddress* ip;

	client=SDLNet_TCP_Accept(server);
	ip=SDLNet_TCP_GetPeerAddress(client);

	SocketClient *c;
	c=new SocketClient();

	if(!c->init(client))
		{
		Log::fail("SocketServer.updateJoin: Nie udalo sie polaczyc z %d.%d.%d.%d", (ip->host)&0xFF, (ip->host>>8)&0xFF, (ip->host>>16)&0xFF, (ip->host>>24)&0xFF);

		delete c;
		}

	int id=clients.size();
	for(unsigned i=0u; i<clients.size(); ++i)
		{
		if(clients[i])
			continue;

		//c->setId(i);
		id=i;
		break;
		}

	if(id<(int)clients.size())
		clients[id]=c;
	else
		clients.push_back(c);
	c->setId(nextid++);

	Event e(Event::TYPE_NETWORK);
	e.type=e.TYPE_NETWORK;
	e.event=e.NETWORK_CONNECT;
	e.network.id=c->getId();
	sendEvent(e);

	Log::info("SocketServer.updateJoin: Polaczono z %d.%d.%d.%d; Klient %u", (ip->host)&0xFF, (ip->host>>8)&0xFF, (ip->host>>16)&0xFF, (ip->host>>24)&0xFF, c->getId());
	}

void SocketServer::updateClients()
	{
	SocketClient *c;

	Event e(Event::TYPE_NETWORK);

	for(unsigned i=0u; i<clients.size(); ++i)
	//for(std::list<SocketClient*>::iterator it=clients.begin(); it!=clients.end(); ++it)
		{
		c=clients[i];//*it;

		if(!c)
			continue;

		e.network.id=c->getId();

		if(c->hasPacket())
			{
			e.event=e.NETWORK_PACKET;
			sendEvent(e);
			}

		if(c->isOk())
			continue;

		e.event=e.NETWORK_DISCONNECT;
		sendEvent(e);

		c->clear();

		delete c;
		clients[i]=NULL;
		//clients.erase(it--);
		}
	}


void SocketServer::send(int id, int type, int length, const char *data)
	{
	SocketClient *c=getClient(id);
	if(c)
		c->send(type, length, data);
	}


SocketClient* SocketServer::getClient(int id)
	{
	for(unsigned i=0u; i<clients.size(); ++i)
		{
		if(!clients[i])
			continue;
		if(clients[i]->getId()==id)
			return clients[i];
		}
	return NULL;
	}


bool SocketServer::isConnected(int id)
	{
	return getClient(id)!=NULL;
	}

bool SocketServer::hasPacket(int id)
	{
	SocketClient *c=getClient(id);
	if(c)
		return c->hasPacket();
	return false;
	}

unsigned SocketServer::getLag(int id)
	{
	SocketClient *c=getClient(id);
	if(c)
		return c->getLag();
	return 0u;
	}

Packet* SocketServer::getPacket(int id)
	{
	SocketClient *c=getClient(id);
	if(c)
		return c->getPacket();
	return NULL;
	}

