/*
 * statemenu.cpp
 *
 *  Created on: 04-08-2013
 *      Author: crm
 */

#include "mainmenu.h"

#include <sstream>

#include "log.h"
#include "ini.h"

#include "render.h"
#include "sound.h"

#include "../gameengine.h"

enum Actions
	{
	NONE,

	NEW_GAME,
	OPTIONS,
	COUNTER,
	EXIT,

	GAME_TOURNAMENT,
	GAME_DUEL,

	OPTION_CONTROLS,
	OPTION_VOLUME,
	OPTION_MUSIC,
	OPTION_RETURN,

	OPTION_CONTROLS_PLAYER_1,
	OPTION_CONTROLS_PLAYER_2,
	OPTION_CONTROLS_PLAYER_3,
	OPTION_CONTROLS_PLAYER_4,
	};

void StateMenu::init()
	{
	Ini lang;
	lang.open("lang.ini", "menu_main");

	m.clear();
	m.init();
	m.getRootLayer()->addOption( new Menu::Option(NEW_GAME, Menu::ENTER, lang.get("start")) );
		m.getOptionByTag(NEW_GAME)->initSubLayer( new Menu::Layer() );
		//m.getOptionByTag(NEW_GAME)->getSubLayer()->addOption( new Menu::Option(GAME_TOURNAMENT, Menu::ACTION, "Turniej") );
		m.getOptionByTag(NEW_GAME)->getSubLayer()->addOption( new Menu::Option(GAME_DUEL, Menu::ACTION, "Duel") );
		m.getOptionByTag(NEW_GAME)->getSubLayer()->addOption( new Menu::Option(NONE, Menu::RETURN, lang.get("return")) );
	m.getRootLayer()->addOption( new Menu::Option(OPTIONS, Menu::ENTER, lang.get("options")) );
		m.getOptionByTag(OPTIONS)->initSubLayer( new Menu::Layer() );
		m.getOptionByTag(OPTIONS)->getSubLayer()->addOption( new Menu::Option(OPTION_CONTROLS, Menu::ENTER, lang.get("options_controls")) );
			m.getOptionByTag(OPTION_CONTROLS)->initSubLayer( new Menu::Layer() );
			m.getOptionByTag(OPTION_CONTROLS)->getSubLayer()->addOption( new Menu::Option(OPTION_CONTROLS_PLAYER_1, Menu::ACTION, lang.get("options_player")+" 1") );
			m.getOptionByTag(OPTION_CONTROLS)->getSubLayer()->addOption( new Menu::Option(OPTION_CONTROLS_PLAYER_2, Menu::ACTION, lang.get("options_player")+" 2") );
			m.getOptionByTag(OPTION_CONTROLS)->getSubLayer()->addOption( new Menu::Option(OPTION_CONTROLS_PLAYER_3, Menu::ACTION, lang.get("options_player")+" 3") );
			m.getOptionByTag(OPTION_CONTROLS)->getSubLayer()->addOption( new Menu::Option(OPTION_CONTROLS_PLAYER_4, Menu::ACTION, lang.get("options_player")+" 4") );
			m.getOptionByTag(OPTION_CONTROLS)->getSubLayer()->addOption( new Menu::Option(OPTION_RETURN, Menu::RETURN, lang.get("return")) );
		m.getOptionByTag(OPTIONS)->getSubLayer()->addOption( new Menu::Option(OPTION_VOLUME, Menu::VALUE, lang.get("options_sound")) );
			m.getOptionByTag(OPTION_VOLUME)->initValue(0.0f, 100.0f, 1.0f);
			m.getOptionByTag(OPTION_VOLUME)->setVal(Sound::getGlobalSoundVolume()*100.0f);
			m.getOptionByTag(OPTION_VOLUME)->setBlock(1/32.0f);
		m.getOptionByTag(OPTIONS)->getSubLayer()->addOption( new Menu::Option(OPTION_MUSIC, Menu::VALUE, lang.get("options_music")) );
			m.getOptionByTag(OPTION_MUSIC)->initValue(0.0f, 100.0f, 1.0f);
			m.getOptionByTag(OPTION_MUSIC)->setVal(Sound::getGlobalMusicVolume()*100.0f);
			m.getOptionByTag(OPTION_MUSIC)->setBlock(1/32.0f);
		m.getRootLayer()->getLastOption()->getSubLayer()->addOption( new Menu::Option(OPTION_RETURN, Menu::RETURN, lang.get("return")) );
	m.getRootLayer()->addOption( new Menu::Option(EXIT, Menu::ACTION, lang.get("exit")) );
	m.block(10.0f);

	//m.print();
	str.init("font/font_02.xml", "", Render::getWindowSize().x, Render::getWindowSize().y);
	str.setAlignCenter();
	str.setAlignMiddle();
	str.setWrapOff();
	redraw();
	}

void StateMenu::update(float dt)
	{
	/**** Sterowanie ****/
	if(!m.isBlocked())
		m.update(dt);
	if(m.needRedraw())
		redraw();

	Sound::setGlobalSoundVolume(m.getOptionByTag(OPTION_VOLUME)->getVal());
	Sound::setGlobalMusicVolume(m.getOptionByTag(OPTION_MUSIC)->getVal());

	/**** Akcje ****/
	if(m.isExited())
		{
		//GameEngine::stop();
		GameEngine::popState();
		return;
		}
	if(m.isBlocked())
		{
		switch(m.getOption()->getTag())
			{
			case GAME_TOURNAMENT:
				Log::info("StateMenu.update: Tryb turniejowy nie jest gotowy."); // todo
				m.unlock();
			break;

			case GAME_DUEL:
				{
				//StateCharacterSelect *s=new StateCharacterSelect();
				//s->init(2);
				//GameEngine::pushState(s);
				//GameEngine::pushState(new StateArenaSelect());
				//GameEngine::pushState(new StateDuel());
				//GameEngine::pushState(new StateDuelPrepare());
				}
			break;

			case EXIT:
				//Log::info("stopping");
				//GameEngine::stop();
				GameEngine::popState();
				return;
			break;

			case OPTION_CONTROLS_PLAYER_1:
			case OPTION_CONTROLS_PLAYER_2:
			case OPTION_CONTROLS_PLAYER_3:
			case OPTION_CONTROLS_PLAYER_4:
				Log::info("StateMenu.update: Zmiana sterowania nie jest gotowa (gracz %d).", m.getOption()->getTag()-OPTION_CONTROLS_PLAYER_1); // todo
				Log::info("StateMenu.update: Pamietaj o: SDL_GetKeyName()");
				m.unlock();
			break;

			default:
				Log::info("StateMenu.update: Nieznana akcja (%d)", m.getOption()->getTag());
				m.unlock();
			break;
			}
		}
	}

void StateMenu::print()
	{
	str.print(Orientation(AVector(0.0f, 0.0f, 0.0f)), 1.0f);
	}

void StateMenu::clear()
	{
	m.clear();
	}


void StateMenu::pause()
	{
	//
	}

void StateMenu::resume()
	{
	m.unlock();
	m.block(10.0f);
	/*if(GameEngine::getState()->getId()==STATE_CHARACTER_SELECT)
		{
		StateCharacterSelect *cs=(StateCharacterSelect *)GameEngine::getState();

		if(cs->isInterrupted())
			Log::info("StateMenu.resume: Przerwano wybor postaci");
		else
			for(int i=0; i<cs->getPlayers(); ++i)
				if(cs->getCharacter(i))
					Log::info("StateMenu.resume: Gracz %d: \"%s\"", i, cs->getCharacter(i)->getName().c_str());
		}*/
	}


void StateMenu::redraw()
	{
	std::stringstream ss;

	const int size=m.getLayer()->getOptionCount();

	for(int i=0; i<size; ++i)
		{
		Menu::Option *opt=m.getLayer()->getOption(i);

		if(i>0)
			ss << '\n';

		const bool cur=m.getOption()==opt;

		if(cur) ss << '[';
		ss << opt->getName();
		if(opt->getType()==Menu::VALUE)
			ss << ": " << opt->getVal();
		if(cur) ss << ']';
		}

	str.setStr(ss.str());
	str.update();
	}
