/*
 * Camera.h
 *
 *  Created on: 07-03-2013
 *      Author: crm
 */

#pragma once

#include "asmmath.h"

#include "orientation.h"

namespace Render
	{
	class Camera: public Orientation
		{
		public:
			enum Type
				{
				ORTHOGRAPHIC,
				PERSPECTIVE,
				GUI,
				};

		protected:

			Type type;

			bool depthbuffer;
			float fov, zoom;
			float cx, cy, cw, ch;	// Viewport
			float cutnear, cutfar;

		public:
			Camera():
				type(ORTHOGRAPHIC), depthbuffer(true), fov(60.0f), zoom(1.0f),
				cx(0), cy(0), cw(0), ch(0), cutnear(1.0f), cutfar(10000.0f) {}
			Camera(Type type, float cx, float cy, float cw, float ch, float cutnear, float cutfar,
					float fov=60.0f, float zoom=1.0f, bool depthbuffer=true):
				type(type), depthbuffer(depthbuffer), fov(fov), zoom(zoom),
				cx(cx), cy(cy), cw(cw), ch(ch), cutnear(cutnear), cutfar(cutfar)
				{
				setType(type);
				}
			virtual ~Camera() {}

			void update();

			inline Type getType() const {return type;}
			inline float getFov() const {return fov;}
			inline float getZoom() const {return zoom;}
			inline float getViewportX() const {return cx;}
			inline float getViewportY() const {return cy;}
			inline float getViewportW() const {return cw;}
			inline float getViewportH() const {return ch;}

			AMatrix getProjection() const;
			AMatrix getModelview() const;// {return getMatrix();}

			void setType(Type s) {type=s;}
			void setFov(float s) {if(fov>0.0f && fov<180.0f) fov=s;}
			void setZoom(float s) {if(zoom>0.0f) zoom=s;}
			void setViewport(float sx, float sy, float sw, float sh) {cx=sx; cy=sy; cw=sw; ch=sh;}

			Orientation getBillboard(const AVector& objpos) const;
			AVector getPointOnScreen(const AVector& objpos) const;
			void getRay(int mx, int my, AVector& raypos, AVector& raydir) const;	// Pozycja myszy, [cx, cw], [xy, ch]
		};
	}

