/*
 * spriteptr.h
 *
 *  Created on: 23-02-2013
 *      Author: crm
 */

#pragma once

#include <string>

namespace Graphics
	{
	class Direction;
	class Animation;
	class Sprite;

	class SpritePtr
		{
		private:
			const Sprite *spr;

			unsigned animID;
			unsigned dirID;

			const Animation *anim;
			const Direction *dir;
			float frame;

			float alpha, scale, angle;

			// todo testowe liczenie liczby pointerow?

		public:
			 SpritePtr();
			 SpritePtr(const Sprite *s);
			 SpritePtr(const SpritePtr& p);
			 SpritePtr(const std::string& name);
			~SpritePtr();

			inline void operator=(const Sprite *s)			{setSprite(s);}
			inline void operator=(const SpritePtr& p)		{setSprite(p.spr);}
			inline bool operator==(const SpritePtr& p) const {return spr==p.spr;}
			inline const Sprite* operator*() const			{return spr;}
			inline const Sprite* operator->() const			{return spr;}
			inline operator bool() const					{return spr!=NULL;}

			void clear();
			void update(float dt);

			bool loop() const;
			bool ended() const;

			inline const Sprite* getSprite() const			{return spr;}
			inline const Animation* getAnim() const			{return anim;}
			inline const Direction* getDir() const			{return dir;}
			inline unsigned getAnimID() const				{return animID;}
			inline unsigned getDirID() const				{return dirID;}
			inline unsigned getFrame() const				{return (unsigned)frame;}

			void getSize(int& gw, int& gh) const;
			int getW() const;
			int getH() const;
			void getHotspot(int& gx, int& gy) const;
			int getHotspotX() const;
			int getHotspotY() const;
			void getActionPoint(int& gx, int& gy) const;
			int getActionPointX() const;
			int getActionPointY() const;
			unsigned getAnimationCount() const;
			unsigned getFrameCount() const;
			float getAlpha() const {return alpha;}
			float getScale() const {return scale;}
			float getAngle() const {return angle;}

			void setSprite(const Sprite *s);
			void setSprite(const SpritePtr& p);
			void setSprite(const std::string& name);
			void setAnim(unsigned sa);
			void setAnim(const std::string& sa);
			void setDir(int sd);
			void setAlpha(float s) {alpha=s;}
			void setScale(float s) {scale=s;}
			void setAngle(float s) {angle=s;}
		};
	}
