/*
 * Render.cpp
 *
 *  Created on: 30-07-2012
 *      Author: crm
 */

#include "render.h"

#include <cmath>
#include <SDL/SDL.h>
#include <GL/glee.h>

#include "asmmath.h"
#include "log.h"
#include "resource.h"

#include "engine.h"
#include "buffer.h"
#include "vertex.h"
#include "graphics/image.h"
#include "graphics/sprite.h"
#include "graphics/spriteptr.h"
#include "camera.h"

// to do sprawdzic czy zmiana kolejnosci bajtow koloru cos zmieni o;	// Zebym ja pamietal o co tu chodzilo...

/******************************************************************************/
/****************** Zmienne ***************************************************/
namespace Render
	{
	bool fullscreen=false;
	SDL_Surface *screen;

	unsigned actTex=0u;			// aktualnie zbindowana tekstura

	float color[4u]={1.0f, 1.0f, 1.0f, 1.0f};

	enum {ALPHA=0, RED=1, GREEN=2, BLUE=3};

	Camera cameras[CAMERA_COUNT];
	Camera *camera=&cameras[CAMERA_DEFAULT];
	}

/******************************************************************************/
/****************** To, co widoczne z zewnatrz ********************************/
/**** Inity i tego typu standardowe sprawy ****/
namespace Render
	{
	bool init(int width, int height, const std::string& title, const std::string& icon)
		{
		Log::debug("Render.init: Startowanie...");

		if(title.length()>0)
			SDL_WM_SetCaption(title.c_str(), NULL);
		else
			SDL_WM_SetCaption("Tajemnicza Zagadka Niepewności", NULL);

		if(icon.length()>0)
			{
			char *data=Resource::load(icon.c_str());
			SDL_RWops *rwops=SDL_RWFromMem(data, Resource::getSize());
			SDL_Surface *img=SDL_LoadBMP_RW(rwops, 1);
			if(img)
				SDL_WM_SetIcon(img, NULL);	// todo zwalnianie surface ikony	// meeeeeh
			if(data)
				delete [] data;
			}

		screen=SDL_SetVideoMode(width, height, 32, SDL_OPENGL|(fullscreen?SDL_FULLSCREEN:0));

		if(!screen)
			{
			Log::fail("Render.init: Nie udalo sie ustawic video mode");
			return false;
			}

		const AVector size=getWindowSize();

		glClearColor(0.0f, 0.5f, 1.0f, 1.0f);
		glDisable(GL_CULL_FACE);
		//glEnable(GL_CULL_FACE);
		glEnable(GL_TEXTURE_2D);
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glEnable(GL_ALPHA_TEST);
		glAlphaFunc(GL_GREATER, 0.0f);
		glDepthFunc(GL_LESS);
		glEnableClientState(GL_VERTEX_ARRAY);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		glEnableClientState(GL_COLOR_ARRAY);

		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();

		int err=glGetError();
		if(err!=GL_NO_ERROR)
			{
			Log::fail("Render.init: Nie udalo sie wystartowac OpenGl'a (%d)", err);
			return false;
			}

		cameras[CAMERA_DEFAULT]=Camera(Camera::PERSPECTIVE, 0.0f, 0.0f, size.x, size.y, 1.0f, 10000.0f, 60.0f, 1.0f, true);
		cameras[CAMERA_UI]=Camera(Camera::GUI, 0.0f, 0.0f, size.x, size.y, -1.0f, 1.0f, 0.0f, 1.0f, false);

		setCamera(&cameras[CAMERA_DEFAULT]);

		return true;
		}

	void update()
		{
		SDL_GL_SwapBuffers();

		glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

		color[ALPHA]=1.0f;
		color[RED]=1.0f;
		color[GREEN]=1.0f;
		color[BLUE]=1.0f;

		camera->update();
		}

	void clear()
		{
		setCamera(&cameras[CAMERA_DEFAULT]);
		}
	}

/**** Sety, gety ****/
namespace Render
	{
	void setColor(float a, float r, float g, float b)
		{
		color[RED]=		r;
		color[GREEN]=	g;
		color[BLUE]=	b;
		color[ALPHA]=	a;
		}

	void setCamera(Camera *scam)
		{
		if(camera==scam)
			return;

		glClear(GL_DEPTH_BUFFER_BIT);	// XXX Zmiana kamery = czyszczenie depth buffera. Tak/Nie?

		camera=scam;
		camera->update();
		}

	void setCamera(CameraID cam)
		{
		setCamera(&cameras[cam]);
		}

	Camera* getCamera()
		{
		return camera;
		}

	bool getFullscreen()
		{
		return fullscreen;
		}

	AVector getWindowSize()
		{
		return AVector(screen->w, screen->h);
		}

	void setFullscreen(bool state)
		{
		fullscreen=state;

		if(screen)
			screen=SDL_SetVideoMode(screen->w, screen->h, 32, SDL_OPENGL|(fullscreen?SDL_FULLSCREEN:0));
		}
	}

/******************************************************************************/
/****************** To, czego glupcy nie dostrzega ****************************/
/**** Takie tam~ ****/
namespace Render
	{
	void bind(unsigned tex)
		{
		if(tex==actTex)
			return;

		actTex=tex;
		glBindTexture(GL_TEXTURE_2D, tex);
		}

	void bind(const Graphics::Image *img)
		{
		bind(img->tex[0]);
		}

	void bind(const Graphics::Sprite *spr)
		{
		if(!spr)
			return;

		const Graphics::Image* img=spr->getImg();

		bind(img->tex[0]);
		}

	void bind(const Graphics::SpritePtr& spr)
		{
		if(!*spr)
			return;

		const Graphics::Image* img=spr->getImg();

		bind(img->tex[0]);
		}
	}

/******************************************************************************/
/****************** Rysowanie *************************************************/
/**** Prymitywy ****/
namespace Render
	{
	void drawLine(const AVector& a, const AVector& b)
		{
		//camera->update();

		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		glDisable(GL_TEXTURE_2D);

		Vertex verts[2]=
				{
				Vertex(a.x, a.y, a.z, 0.0f, 0.0f, 255.0f*color[ALPHA], (unsigned char)(255.0f*color[RED]), (unsigned char)(255.0f*color[GREEN]), (unsigned char)(255.0f*color[BLUE])),
				Vertex(b.x, b.y, b.z, 0.0f, 0.0f, 255.0f*color[ALPHA], (unsigned char)(255.0f*color[RED]), (unsigned char)(255.0f*color[GREEN]), (unsigned char)(255.0f*color[BLUE])),
				};

		glVertexPointer		(3, GL_FLOAT,			sizeof(Vertex),	&verts[0].x);
		//glTexCoordPointer	(2, GL_FLOAT,			sizeof(Vertex),	&verts[0].tx);
		glColorPointer		(4, GL_UNSIGNED_BYTE,	sizeof(Vertex),	&verts[0].col);
		glDrawArrays(GL_LINES, 0, 2);

		glEnable(GL_TEXTURE_2D);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		}

	void drawTriangle(const AVector& a, const AVector& b, const AVector& c)
		{
		//camera->update();

		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		glDisable(GL_TEXTURE_2D);

		Vertex verts[3]=
				{
				Vertex(a.x, a.y, a.z, 0.0f, 0.0f, 255.0f*color[ALPHA], (unsigned char)(255.0f*color[RED]), (unsigned char)(255.0f*color[GREEN]), (unsigned char)(255.0f*color[BLUE])),
				Vertex(b.x, b.y, b.z, 0.0f, 0.0f, 255.0f*color[ALPHA], (unsigned char)(255.0f*color[RED]), (unsigned char)(255.0f*color[GREEN]), (unsigned char)(255.0f*color[BLUE])),
				Vertex(c.x, c.y, c.z, 0.0f, 0.0f, 255.0f*color[ALPHA], (unsigned char)(255.0f*color[RED]), (unsigned char)(255.0f*color[GREEN]), (unsigned char)(255.0f*color[BLUE])),
				};

		glVertexPointer		(3, GL_FLOAT,			sizeof(Vertex),	&verts[0].x);
		//glTexCoordPointer	(2, GL_FLOAT,			sizeof(Vertex),	&verts[0].tx);
		glColorPointer		(4, GL_UNSIGNED_BYTE,	sizeof(Vertex),	&verts[0].col);
		glDrawArrays(GL_TRIANGLES, 0, 3);

		glEnable(GL_TEXTURE_2D);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		}

	void drawRect(const AVector& a, const AVector& b, const AVector& c, const AVector& d)
		{
		//camera->update();

		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		glDisable(GL_TEXTURE_2D);

		Vertex verts[4]=
				{
				Vertex(a.x, a.y, a.z, 0.0f, 0.0f, 255.0f*color[ALPHA], (unsigned char)(255.0f*color[RED]), (unsigned char)(255.0f*color[GREEN]), (unsigned char)(255.0f*color[BLUE])),
				Vertex(b.x, b.y, b.z, 0.0f, 0.0f, 255.0f*color[ALPHA], (unsigned char)(255.0f*color[RED]), (unsigned char)(255.0f*color[GREEN]), (unsigned char)(255.0f*color[BLUE])),
				Vertex(c.x, c.y, c.z, 0.0f, 0.0f, 255.0f*color[ALPHA], (unsigned char)(255.0f*color[RED]), (unsigned char)(255.0f*color[GREEN]), (unsigned char)(255.0f*color[BLUE])),
				Vertex(d.x, d.y, d.z, 0.0f, 0.0f, 255.0f*color[ALPHA], (unsigned char)(255.0f*color[RED]), (unsigned char)(255.0f*color[GREEN]), (unsigned char)(255.0f*color[BLUE])),
				};

		glVertexPointer		(3, GL_FLOAT,			sizeof(Vertex),	&verts[0].x);
		//glTexCoordPointer	(2, GL_FLOAT,			sizeof(Vertex),	&verts[0].tx);
		glColorPointer		(4, GL_UNSIGNED_BYTE,	sizeof(Vertex),	&verts[0].col);
		glDrawArrays(GL_QUADS, 0, 4);

		glEnable(GL_TEXTURE_2D);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		}

	void drawTexturedTriangle(const AVector& a, const AVector& b, const AVector& c, float ax, float ay, float bx, float by, float cx, float cy)
		{
		//camera->update();

		Vertex verts[3]=
				{
				Vertex(a.x, a.y, a.z, ax, ay, 255.0f*color[ALPHA], (unsigned char)(255.0f*color[RED]), (unsigned char)(255.0f*color[GREEN]), (unsigned char)(255.0f*color[BLUE])),
				Vertex(b.x, b.y, b.z, bx, by, 255.0f*color[ALPHA], (unsigned char)(255.0f*color[RED]), (unsigned char)(255.0f*color[GREEN]), (unsigned char)(255.0f*color[BLUE])),
				Vertex(c.x, c.y, c.z, cx, cy, 255.0f*color[ALPHA], (unsigned char)(255.0f*color[RED]), (unsigned char)(255.0f*color[GREEN]), (unsigned char)(255.0f*color[BLUE])),
				};

		glVertexPointer		(3, GL_FLOAT,			sizeof(Vertex),	&verts[0].x);
		glTexCoordPointer	(2, GL_FLOAT,			sizeof(Vertex),	&verts[0].tx);
		glColorPointer		(4, GL_UNSIGNED_BYTE,	sizeof(Vertex),	&verts[0].col);
		glDrawArrays(GL_TRIANGLES, 0, 3);
		}

	void drawTexturedRect(const AVector& a, const AVector& b, const AVector& c, const AVector& d, float ax, float ay, float bx, float by, float cx, float cy, float dx, float dy)
		{
		//camera->update();

		Vertex verts[4]=
				{
				Vertex(a.x, a.y, a.z, ax, ay, 255.0f*color[ALPHA], (unsigned char)(255.0f*color[RED]), (unsigned char)(255.0f*color[GREEN]), (unsigned char)(255.0f*color[BLUE])),
				Vertex(b.x, b.y, b.z, bx, by, 255.0f*color[ALPHA], (unsigned char)(255.0f*color[RED]), (unsigned char)(255.0f*color[GREEN]), (unsigned char)(255.0f*color[BLUE])),
				Vertex(c.x, c.y, c.z, cx, cy, 255.0f*color[ALPHA], (unsigned char)(255.0f*color[RED]), (unsigned char)(255.0f*color[GREEN]), (unsigned char)(255.0f*color[BLUE])),
				Vertex(d.x, d.y, d.z, dx, dy, 255.0f*color[ALPHA], (unsigned char)(255.0f*color[RED]), (unsigned char)(255.0f*color[GREEN]), (unsigned char)(255.0f*color[BLUE])),
				};

		glVertexPointer		(3, GL_FLOAT,			sizeof(Vertex),	&verts[0].x);
		glTexCoordPointer	(2, GL_FLOAT,			sizeof(Vertex),	&verts[0].tx);
		glColorPointer		(4, GL_UNSIGNED_BYTE,	sizeof(Vertex),	&verts[0].col);
		glDrawArrays(GL_QUADS, 0, 4);
		}
	}

/**** Sprajty ****/
namespace Render
	{
	void print(const Graphics::Image *img, const Orientation& orientation, float x, float y, float w, float h, float alpha)
		{
		const float iw=img->w;
		const float ih=img->h;
		const AMatrix m=AMatrixTranspose(orientation.getMatrix());
		const AVector s=orientation.getScale();
		const AVector p=orientation.getPos();

		Vertex verts[4]=
			{
			Vertex(    0, h*(s.y-1), 0,  x   /iw,  y   /ih, 255.0f*color[ALPHA]*alpha, (unsigned char)(255.0f*color[RED]), (unsigned char)(255.0f*color[GREEN]), (unsigned char)(255.0f*color[BLUE])),
			Vertex(w*s.x, h*(s.y-1), 0, (x+w)/iw,  y   /ih, 255.0f*color[ALPHA]*alpha, (unsigned char)(255.0f*color[RED]), (unsigned char)(255.0f*color[GREEN]), (unsigned char)(255.0f*color[BLUE])),
			Vertex(w*s.x,-h,         0, (x+w)/iw, (y+h)/ih, 255.0f*color[ALPHA]*alpha, (unsigned char)(255.0f*color[RED]), (unsigned char)(255.0f*color[GREEN]), (unsigned char)(255.0f*color[BLUE])),
			Vertex(    0,-h,         0,  x   /iw, (y+h)/ih, 255.0f*color[ALPHA]*alpha, (unsigned char)(255.0f*color[RED]), (unsigned char)(255.0f*color[GREEN]), (unsigned char)(255.0f*color[BLUE])),
			};

		bind(img);

		glVertexPointer		(3, GL_FLOAT,			sizeof(Vertex),	&verts[0].x);
		glTexCoordPointer	(2, GL_FLOAT,			sizeof(Vertex),	&verts[0].tx);
		glColorPointer		(4, GL_UNSIGNED_BYTE,	sizeof(Vertex),	&verts[0].col);

		glPushMatrix();
		glTranslatef(p.x, p.y, p.z);
		//glScalef(s.x, s.y, s.z);
		glMultMatrixf((float*)&m);
		glDrawArrays(GL_QUADS, 0, 4);
		glPopMatrix();
		}

	void print(const Graphics::Image *img, const Orientation& orientation, float alpha)
		{
		print(img, orientation, 0, 0, img->w, img->h, alpha);
		}

	void print(const Graphics::Sprite *spr, const Orientation& orientation, unsigned animation, unsigned direction, unsigned frame, float alpha)
		{
		const Graphics::Animation& a=spr->getAnim(animation);
		const Graphics::Direction& d=a.getDir((Graphics::Directions)direction);
		const Graphics::Frame& f=d.getFrame(frame);

		const AVector pos=orientation.getPos()+orientation.getForward()*f.spoty-orientation.getRight()*f.spotx;

		print(spr->getImg(), Orientation(pos, orientation), f.x, f.y, f.w, f.h, alpha);
		}

	void print(const Graphics::SpritePtr& spr, const Orientation& orientation, float alpha, bool rotate)
		{
		unsigned dir=spr.getDirID();

		// Zakladam, ze jest to billboard - inaczej moze wyjsc nieciekawie~
		if(rotate)
			{
			const AVector& f=AVectorNormalize(orientation.getPos()-camera->getPos());//camera->getForward();
			const AVector& forward=AVectorNormalize(AVector(f.x, f.y, 0.0f));
			const float fu=forward|AVector(0, -1, 0);
			const float fr=forward|AVector(1, 0, 0);
			const bool frgt=fr>0.0f;

			const float ang=acosf(fu)*180.0f/M_PI;
			const float tmp=((360-ang)*frgt+(ang)*!frgt + 45.0f/2.0f);

			dir+=8-(int)(tmp/45.0f);

			if(dir>=Graphics::COUNT) dir-=Graphics::COUNT;
			else if(dir<0)			 dir+=Graphics::COUNT;
			}

		print(*spr, orientation, spr.getAnimID(), dir, spr.getFrame(), alpha);
		}

	void print(Render::Buffer& buff, const Orientation& orientation, float alpha)
		{
		const AMatrix m=AMatrixTranspose(orientation.getMatrix());
		const AVector s=orientation.getScale();
		const AVector p=orientation.getPos();
		const Vertex *verts=buff.getVerts();
		const Vertex dummy; // Zeby offsetof sie nie plul

		buff.bind();

		/*if(buff.getId())
			{
			glBindBuffer(GL_ARRAY_BUFFER, buff.getId());

			if(!buff.isEmpty())
				{
				//Log::ok("Render.print: Wysylanie danych, buff %u, count %u", buff.getId(), buff.getCount());
				glBufferData(GL_ARRAY_BUFFER, buff.getCount()*sizeof(Vertex), verts, GL_STATIC_DRAW);
				buff.flush();
				}

			glVertexPointer		(3, GL_FLOAT,			sizeof(Vertex), (void *)offsetof(Vertex, x));
			glTexCoordPointer	(2, GL_FLOAT,			sizeof(Vertex), (void *)offsetof(Vertex, tx));
			glColorPointer		(4, GL_UNSIGNED_BYTE,	sizeof(Vertex), (void *)offsetof(Vertex, col));
			}
		else*/
			{
			glVertexPointer		(3, GL_FLOAT,			sizeof(Vertex),	&verts[0].x);
			glTexCoordPointer	(2, GL_FLOAT,			sizeof(Vertex),	&verts[0].tx);
			glColorPointer		(4, GL_UNSIGNED_BYTE,	sizeof(Vertex),	&verts[0].col);
			}

		glPushMatrix();
		glTranslatef(p.x, p.y, p.z);
		//glScalef(s.x, s.y, s.z);
		glMultMatrixf((float*)&m);
		glDrawArrays(GL_QUADS, 0, buff.getCount());
		glPopMatrix();

		glBindBuffer(GL_ARRAY_BUFFER, 0);
		}
	}
