/*
 * log.cpp
 *
 *  Created on: 23-12-2011
 *      Author: crm
 */

#define LOG_CPP

#include <cstdio>
#include <cstdarg>
#include <ctime>

#include "log.h"

//#include <pthread.h>
#include <SDL/SDL_mutex.h>

namespace Log
	{
	void date();
	void flush();

	int logLvl=ALL;
	int lastDay=0;
	int logDepth=0;

	FILE *out=stdout;
	va_list args;

	//pthread_mutex_t mutex=NULL;
	SDL_mutex *mutex=NULL;
	}

void Log::init()
	{
	//pthread_mutex_init(&Log::mutex, NULL);
	mutex=SDL_CreateMutex();
	}

void Log::clear()
	{
	if(Log::mutex)
		//pthread_mutex_destroy(&Log::mutex);
		SDL_DestroyMutex(mutex);

	Log::mutex=NULL;
	}

void lock()
	{
	if(!Log::mutex)
		return;

	//pthread_mutex_lock(&Log::mutex);
	SDL_mutexP(Log::mutex);
	}

void unlock()
	{
	if(!Log::mutex)
		return;

	//pthread_mutex_unlock(&Log::mutex);
	SDL_mutexV(Log::mutex);
	}

void Log::date()
	{
	static char date[64];
	static time_t dtime;
	static tm *ddate;

	dtime=time(NULL);
	ddate=localtime(&dtime);

	if(lastDay!=ddate->tm_mday)
		{
		strftime(date, 64, "%Y-%m-%d %H:%M:%S", ddate);
		lastDay=ddate->tm_mday;
		}
	else
		strftime(date, 64, "%H:%M:%S", ddate);
	fprintf(out, "[%s:%03ld] ", date, clock()%1000);

	for(int i=0; i<logDepth; ++i)
		fprintf(out, " ");
	}

void Log::flush()
	{
	fflush(out);
	}

void Log::info(/*const char *loc, */const char *fmt, ...)
	{
	if(!(logLvl&INFO))
		return;
	lock();
	date();
	fprintf(out, "INFO: ");

	va_start(args, fmt);
	vfprintf(out, fmt, args);
	va_end(args);

	fprintf(out, "\n");
	flush();
	unlock();
	}

void Log::fail(/*const char *loc, */const char *fmt, ...)
	{
	if(!(logLvl&FAIL))
		return;
	lock();
	date();
	fprintf(out, "FAIL: ");

	va_start(args, fmt);
	vfprintf(out, fmt, args);
	va_end(args);

	fprintf(out, "\n");
	flush();
	unlock();
	}

void Log::ok(/*const char *loc, */const char *fmt, ...)
	{
	if(!(logLvl&OK))
		return;
	lock();
	date();
	fprintf(out, "OK  : ");

	va_start(args, fmt);
	vfprintf(out, fmt, args);
	va_end(args);

	fprintf(out, "\n");
	flush();
	unlock();
	}

void Log::debug(/*const char *loc, */const char *fmt, ...)
	{
	if(!(logLvl&DEBUG))
		return;
	lock();
	date();
	//fprintf(out, "DEBUG ");
	//fprintf(out, "INFO: ");
	fprintf(out, "DBG : ");

	va_start(args, fmt);
	vfprintf(out, fmt, args);
	va_end(args);

	fprintf(out, "\n");
	flush();
	unlock();
	}

void Log::push()
	{
	if(logDepth<10)
		++logDepth;
	}

void Log::pop()
	{
	if(logDepth>0)
		--logDepth;
	}

void Log::close()
	{
	lock();
	if(out!=NULL && out!=stdout)
		fclose(out);
	unlock();
	}

void Log::setOut(FILE *sf)
	{
	lock();
	if(!sf)
		fail("Log.setOut: NULL");
	else
		out=sf;
	unlock();
	}

void Log::setLogLvl(int flag)
	{
	logLvl=flag;
	}

int Log::getLogLvl()
	{
	return logLvl;
	}


