/*
 * lua2.h
 *
 *  Created on: 02-10-2013
 *      Author: crm
 */

#pragma once

#include <string>

struct lua_State;

class Lua
	{
	public:
		typedef int (*LuaFunction)(lua_State *);

		bool addTableStack(const std::string& name);

	private:
		bool error();
		//bool addTableStack(const std::string& name);

		lua_State *state;

		int status;
		int args;

		std::string path;

	public:
		 Lua(): state(0), status(0), path("script") {}
		~Lua() {clear();}

		bool init();
		void clear();

		bool open(const std::string& name);
		bool call(const std::string& name);
		bool call(int ref);
		bool parse(const std::string& script);

		inline bool operator() (const std::string& name) {return call(name);}
		inline bool operator= (const std::string& line) {return parse(line);}
		inline operator bool () {return isOk();}

		inline bool isOk() {return (status==0);}

		void push(int val);
		void push(float val);
		void push(const std::string& str);
		void pop();

		//void addTab(const std::string& path);
		void addGlobal(const std::string& name, int val);
		void addGlobal(const std::string& name, float val);
		void addGlobal(const std::string& name, const std::string& str);
		void addGlobal(const std::string& name, void *ptr);
		void addGlobal(const std::string& name, LuaFunction fptr);

		void addRegistry(const std::string& name, int val);
		void addRegistry(const std::string& name, float val);
		void addRegistry(const std::string& name, const std::string& str);
		void addRegistry(const std::string& name, void *ptr);

		bool functionExists(const std::string& name);
		void createFunction(const std::string& name);
	};

